jQuery(document).ready(function ($) {
  // API REF
  var TABLE_REFS = getTableRefs($);
  var LOWEST_DESKTOP_RESOLUTION = 1023;
  var RECORDS_COUNT_CLASSES = 'badge-danger badge-primary';
  var DEFAULT_RECORDS_COUNT_CLASS = 'badge-primary';
  var RECORDS_COUNT_HIGHLIGHT_CLASS = 'badge-danger';

  // Global elements
  var $spinner = $('.spinner');
  var $lastUpdatedOn = $('#dv2LastUpdatedOn');
  var $monitorSummary = $('.monitor-summary');
  var $currentTime = $('#dvCurrentTime');
  var $configurationMenu = $('#configMenu, #topConfigMenuDesktop');
  var $APILogsMenu = $('#APILogsMenuMobile, #APILogsMenuDesktop');
  var $statsMenu = $('#StatsDesktop');
  var $uptime = $('#dvUptime');
  var $stopBtn = $('#btnStopContainer').find('button');
  var $contentPage = $('#contentPage');
  var $topNav = $('#topNav');
  var $testModeContainer = $('#testModeContainerRibbon');
  var $subHeader = $('#subHeader');
  var $topbar = $('.topbar');
  var $footer = $('footer');
  var $buySellButton = $('.buy-or-sell-btn');

  /**
   * Arrays to use api/v2/
   */
  let possibleBuyLog = [];
  let pairsLog = [];
  let watchModeLog = [];
  let dcaLog = [];
  let pendingLog = [];
  let buyLog = [];
  let salesLog = [];
  let dustLog = [];
  let reservedCoins = [];
  let reversalCoins = [];
  let miscLog = [];

  let statsLog = [];
  let propertiesLog = [];
  let notificationsLog = [];
  let heartbeats = [];
  let accountLog = {};

  let currentMarketPrecision = PBConstants.DEFAULT_PRECISION;

  // Store for the server data.
  var serverData = {};
  var editor = '';
  var APILogsEditor = '';
  var PTFConfigEditorApp = '';
  var PTFConfigEditorHost = '';
  var PTFeederEditor = '';
  var tableRecordsCount = {};
  var routes, Service, NormalizerService, ConfigurationService;
  var currenciesLocalStorageKey = 'currencies';
  var currenciesAPICalledTimeLocalStorageKey = 'currenciesAPICalledTime';
  var updatingSummaryClass = 'js-updating-summary';
  var serverDataTimeout;
  var marketIcons = {
    BTC: 'fa fa-btc',
    USDT: 'fa fa-usd',
    BNB: 'fa fa-dot-circle-o',
    XMR: 'cf cf-xmr',
    ETH: 'cf cf-eth'
  };
  var topbarMargin = 5;

  // API calls should happen after localization and loading translations which will happen in the below function call
  // otherwise translated messages will not be available in API call back functions.
  Translations.init();

    function loadPage(isPageLoad) {
    // make the ajax call to get the data
      /**
       * Api/v2/ routes getters
       */
      getAccountLog();
      getPossibleBuyLog();
      getPairsLog();
      getWatchModeLog();
      getDcaLog();
      getPendingLog();
      getBuyLog();
      getSalesLog();
      getDustLog();

      getReservedCoins();
      getReversalCoins();
      getStatsLog();
      getPropertiesLog();
      getNotificationsLog();
      getHeartbeatsLog();
      getMiscLog();

      /**
       * End of api/v2 routes
       */


    getServerData(function (data) {
      data.stats = NormalizerService.normalizeStats(data);
      // data.currentMarketPrecision = Service.getCurrentMarketPrecision();
      data = NormalizerService.segregateNotifications(data);
      data.normalizedMarket = NormalizerService.normalizeMarket(data.market);

      // Call coin market API function only on page load.
      if (isPageLoad) {
        CoinMarket.callCoinMarket(data);
        APILogs.callAPILogs();
        AppFeed.callFeed(data);
      }
      data = CoinMarket.getCoinMarketData(data);
      serverData = data;
      TABLE_REFS.cbUpdateDtCache(data);
      /**
       * V2 ready routes
       */
      currentMarketPrecision = Service.getCurrentMarketPrecision();

      showOrHideConfigurationMenu();
      showOrHideAPILogsMenu(); // Not doing anything at glance - Not a menu point anymore. TODO look at
      // Add api market value.
      addMarketValue();
      hidePublishButton();
      updateUptime();
      updateStatsBasedOnServerData();
      updatePTTrend();
      /**
       * End of v2 ready
       */

      // update the UI elements
      // console.log(data)
      updateLayoutItems(data);

      // Enable stop,logout and switchery buttons.
      var settingsDetails = NormalizerService.settingsDetails(data);
      DomHelper.toggleBtns(true, settingsDetails);

      Translations.loadCurrentLanguage(data);

      // Display the notification and remove it from serverData since we dont want so many
      // unnecessary objects in memory.
      AppNotification.display(data.nrmlsdNotifications, data.timeZoneOffset);
      AppSignal.displaySignals(JSON.parse(data.settings.myProducts), data.timeZoneOffset);

      delete data.notifications;
      delete data.nrmlsdNotifications;
      delete data.heartBeats;

      // routes variable will not have a property with querystring.
      // if(LS.gI(NormalizerService.generateLocalStorageKey('savePath'))){

      var currentPath = window.history.state ? window.history.state.path.split('?')[0] : 'monitoring';

      // Not sure why this is needed because it triggers the callback for this page twice.
      if(currentPath != 'ptn-settings'){
          RouteHelper.showCurrentPage(routes, currentPath, isPageLoad, data);
      }
      RouteHelper.setServerData(data);
      setMainContentMarginAndHeight();
      //initializeStats(data);
    });
  }

  function updatePTTrend() {
      if(!propertiesLog.currentTrend) {
        return;
      }
      let element = $('#ptTrend');

      if(!propertiesLog.currentTrend) {
        element.addClass('hide');
        return;
      }

      element.removeClass('hide');

      if(propertiesLog.currentTrend === 'BULL') {
        element.find('i').removeClass('mdi-trending-down');
        element.find('i').removeClass('mdi-trending-neutral');
        element.find('i').addClass('mdi-trending-up');
      } else if(propertiesLog.currentTrend === 'BEAR') {
        element.find('i').removeClass('mdi-trending-neutral');
        element.find('i').removeClass('mdi-trending-up');
        element.find('i').addClass('mdi-trending-down');
      } else {
        element.find('i').removeClass('mdi-trending-up');
        element.find('i').removeClass('mdi-trending-down');
        element.find('i').addClass('mdi-trending-neutral');
      }
      element.attr('title', propertiesLog.currentTrendValues);
  }

  function getCurrencies() {
    var normalizedData = null;
    var currenciesTmp = localStorage.getItem(NormalizerService.generateLocalStorageKey(currenciesLocalStorageKey));
    var currenciesAPICalledTime = +localStorage.getItem(
      NormalizerService.generateLocalStorageKey(
        currenciesAPICalledTimeLocalStorageKey
      )
    );
    var currentEpochTime = new Date().getTime();

    // Check local storage has currencies value and currencies API called time,
    // if above two values are there and currencies API called time didn't cross 24 hours then consider local storages values.
    if (currenciesTmp && currenciesAPICalledTime) {
      if (
        currentEpochTime - currenciesAPICalledTime <
        PBConstants.CURRENCY_API_CALL_FREQUENCY
      ) {
        serverData = NormalizerService.normalizeCurrencies(
          serverData,
          JSON.parse(currenciesTmp)
        );
        normalizedData = serverData;
        ajaxStopCb();
        return normalizedData;
      }
    }

    // If localstorage doesn't have currencies or currencies API called time
    // (or) currencies API called time crossed 24 hours then call currencies API.
    Service.getCurrencies()
      .done(function (response) {
        if (response) {
          localStorage.setItem(
            NormalizerService.generateLocalStorageKey(currenciesLocalStorageKey),
            JSON.stringify(response)
          );
          localStorage.setItem(
            NormalizerService.generateLocalStorageKey(
              currenciesAPICalledTimeLocalStorageKey
            ),
            currentEpochTime
          );
          serverData = NormalizerService.normalizeCurrencies(
            serverData,
            response
          );
          normalizedData = serverData;
        }
      })
      .fail(function () {
        toastr.info(i18next.t('constantMsgs.ajaxError'));
      })
      .always(function () {
        ajaxStopCb();
      });
      return normalizedData;
  }

  // Triggers on language file load
  $(document).on('evt.after-lang-file-load', function () {
    // Initialize the services.
    Service = Services(
      PBConstants.BASE_API_URL,
      PBConstants.MARKET_CAP_API_URL,
      PBConstants.CURRENCIES_API_URL,
      PBConstants.GLOBAL_MARKET_CAP_API_URL
    );
    NormalizerService = Service.Normalizer;
    ConfigurationService = Service.Configurations;

    routes = {
      'public-configs': {
        template: 'tmplPublicConfig',
        heading: i18next.t('settingInfo.changeConfig.label'),
        refresh: false,
       },
      'change-password':{
        template: 'tmplchangePassword',
        heading: i18next.t('settingInfo.chngePswrd.label'),
        refresh: true
      },
      'account': {
        template: 'tmplAccount',
        heading: i18next.t('sidebar.account'),
        callback: cbAccount,
        refresh: false
      },
      'stats': {
        template: 'tmplStats',
        heading: i18next.t('sidebar.stats'),
        callback: cbStats,
        refresh: true
      },
      monitoring: {
        template: 'tmplMonitoring',
        heading: i18next.t('sidebar.monitoring'),
        callback: cbMonitoring,
        refresh: true
      },
      'totals-log': {
        template: 'tmplTotalLog',
        heading: i18next.t('pageHeadings.totalLog'),
        refresh: true
      },
      'active-config': {
        template: 'tmplActiveConfig',
        heading: i18next.t('pageHeadings.configuration'),
        callback: cbLoadConfig,
        refresh: false
      },
      'other-configs': {
        template: 'tmplOtherConfigs',
        heading: i18next.t('pageHeadings.configuration'),
        callback: cbLoadConfig,
        refresh: false
      },
      'hot-config': {
        template: 'tmplHotConfig',
        heading: i18next.t('sidebar.hotconfig'),
        callback: cbHotConfig,
        refresh: false
      },
      'possible-buys-log': {
        template: 'tmplPossibleBuys',
        callback: cbPossibleBuys,
        heading: i18next.t('sidebar.possibleBuyLog'),
        refresh: true
      },
      'pairs-log': {
        template: 'tmplPairsLog',
        callback: cbPairLog,
        heading: i18next.t('sidebar.pairsLog'),
        refresh: true
      },
      'pending-log': {
        template: 'tmplPendingLog',
        callback: cbPendingLog,
        heading: i18next.t('sidebar.pendingLog'),
        refresh: true
      },
      'news': {
        template: 'tmplNews',
        callback: cbNews,
        heading: i18next.t('sidebar.news'),
        refresh: false
      },
      'buy-log': {
        template: 'tmplBuyLog',
        callback: cbBuyLog,
        heading: i18next.t('sidebar.buyLog'),
        refresh: true
      },
      'dca-log': {
        template: 'tmplDcaLog',
        callback: cbDcaLog,
        heading: i18next.t('sidebar.dcaLog'),
        refresh: true
      },
      'sales-log': {
        template: 'tmplSalesLog',
        callback: cbSalesLog,
        heading: i18next.t('sidebar.salesLog'),
        refresh: true
      },
      'misc-log': {
        template: 'tmplDustLog',
        callback: cbDustLog,
        heading: i18next.t('sidebar.dustLog'),
        refresh: true
      },
      'ptfeeder': {
        template: 'tmplPTFeeder',
        callback: cbPTFeeder,
        heading: i18next.t('sidebar.ptfeeder'),
        refresh: false,
      }
    };
    addUTCTimer();
    RouteHelper.init(routes);
    AppNotification.init();
    AppFeed.init();
    AppSignal.init();
    Settings.init();
    CoinMarket.init();
    APILogs.init();
    loadPage(true);
    Settings.setTheme(true);
    ConfigurationService.setToastrOptions();
    configInit();
    showCurrentPage();
    clickEventHandlers();
    highlightCurrentPageinTopMenu();
    intializeTooltips();
    handleDustValueTooltips();
    Translations.loadTimeAgoTranslations();
    reloadPageOnInactivity(PBConstants.INACTIVITY_REFRESH_TIMER);
    /* configInit() does everything needed for config GUI to load*/

  });

  $(document).on('evt.render-signals-dropdown', function () {
    // Calling displaySignal when we click on view all/ view less button in signals dropdown.
    AppSignal.displaySignals(JSON.parse(serverData.settings.myProducts), serverData.timeZoneOffset);
  });

  function updateShortingStatus(settingsData) {
    if (settingsData.shorting) {
      $('.shorting-status').attr('style', 'display: inline');
    } else {
      $('.shorting-status').attr('style', 'display: none');
    }
  }

  function highlightCurrentPageinTopMenu() {
    var currentPath = window.history.state ? window.history.state.path : 'monitoring';
    if (currentPath) {
      var $currentAnchorLink = $('[href="' + currentPath + '"]');
      var $anchorLinkParents = $currentAnchorLink.parents();
      $anchorLinkParents.eq(0).addClass('active');
      $anchorLinkParents.eq(2).addClass('active');
      $anchorLinkParents.eq(4).addClass('active');
    }
  }

  addEventHandler();
  handleWindowResize();
  scrollTopInLowerResolutions();
  handleConfigurationUnsavedChanges();
  savePath();
  function addEventHandler() {
    $(document).on('submit','#saveAdmin,#saveMonitor', function (event) {
      event.preventDefault();
      var formData = $(this).serialize();
      if($(event.target).attr('id') === 'saveAdmin'){
        formData = formData +'&saveAdmin';
      } else {
        formData = formData +'&saveMonitor';
      }
      Service.savePassword(formData);
    });

    $('#apiSellOnlyMode').on('click', function (event) {
      event.stopPropagation();
      event.preventDefault();
      if ((serverData.settings.role).toUpperCase() === PBConstants.ENABLE_SETTINGS_BTNS_ROLE){
        return;
      }

      // To switch SOM when we click it in footer section.
      let currentState = $('#apiSellOnlyMode').text();
      currentState = currentState === 'true' ? true : false;
      Service.saveSOMO(!currentState).done(function (
        data,
        textStatus,
        jqXHR
      ) {
        if (jqXHR.status === 204) {
          toastr.success(i18next.t('constantMsgs.somoSuccess'));
        } else {
          toastr.info(i18next.t('constantMsgs.ajaxError'));
        }
      });
      if ($('#somoSwitcher').length) {

        // To switch SOMO switcher when in accounts page.
        $('#somoSwitcher')[0].checked = !currentState;
        $('#somoSwitcher')[0].value = !currentState;
      }
    });

    $(document).on('evt.before-page-load', function () {
      setMainContentMarginAndHeight();
      TABLE_REFS.destroyTable();
    });

    $(document).on('evt.add-market-value', function () {
      addMarketValue();
    });

    $('#dvPageContent').on('click', '#aceEditorSearch', function () {
      editor.execCommand('find');
    });
    $(document).on('evt.load-page-data', function () {
      loadPage(false);
    });
    $(document).on('evt.set-config-editor-height', function () {
      setConfigurationContainerHeight();
    });
  }

  /**
   * Topbar and footer are position fixed and height of topbar and footer will differ from one resolution to another resolution.
   * We also have to change margin-top and height of main content according to the afbove comment changes.
   *
   */
  function setMainContentMarginAndHeight() {
    // We have added fixed subheader only in desktop resolutions.
    if (window.innerWidth > LOWEST_DESKTOP_RESOLUTION) {
      // Calculate topbar height and add margintop to mail content.
      var topbarHeight =
        $subHeader.innerHeight() + $topbar.innerHeight() - topbarMargin;
      var contentPageHeight = topbarHeight + $footer.innerHeight();
      $contentPage
        .css('margin-top', topbarHeight);
      // Scrollbar should appear to config content not to main content in config section.
      if (window.history.state.path === 'config') {
        $contentPage.css('height', 'auto');
      } else {
        $contentPage.css('height', 'calc(100vh - ' + contentPageHeight + 'px)');
      }
    } else {
      $contentPage.css('margin-top', 0).css('height', 'auto');
    }
  }

  function hidePublishButton() {
    /* To display publish button when all of the following condition is true.
     * 1. isAllowedToPublish is true.
     * 2. passwordSet is true.
     * 3. Role is not 'M'.
     */
    if (propertiesLog.hasOwnProperty('isAllowedToPublish') &&
        propertiesLog.isAllowedToPublish.toString().toLowerCase() === 'true' &&
        propertiesLog.passwordSet &&
        propertiesLog.role !== PBConstants.ENABLE_SETTINGS_BTNS_ROLE) {
      $('.share-config').removeClass('hide');
    } else {
      $('.share-config').addClass('hide');
    }
  }

  function addMarketValue() {
    // Add api market value.
    // Adding API data into DOM only when it is available.
    if (propertiesLog.hasOwnProperty('currency')) {
      $('.api-market').html(miscLog.market);
      $('.api-currency').html(propertiesLog.currency);
      $('#nSelectedConfig').html(propertiesLog.activeConfig);
      $('.current.active-config').html(propertiesLog.activeConfig);
    }
  }

  // We have to add market values for child rows which will appear on click on + icon.
  $('#dvPageContent').on(
    'click',
    '.collapsed tbody tr td:first-child',
    function () {
      addMarketValue();
    }
  );

  /**
   * If a user tries to close the tab without saving then we should inform them.
   */
  function handleConfigurationUnsavedChanges() {
    window.addEventListener('beforeunload', function (event) {
      if (isConfigurationSaved()) {
        event.preventDefault();
      }
    });

    window.unload = function (e) {
      if (isConfigurationSaved()) {
        var dialogText = 'Not saved';
        e.returnValue = dialogText;
        return dialogText;
      }
    };
  }

  function savePath() {
    const defaulpath = 'monitoring';
    var validPaths = [
      'possible-buys-log',
      'pairs-log',
      'dca-log',
      'buy-log',
      'sales-log',
      'misc-log'
    ];
    window.addEventListener('beforeunload', function(event) {
      var currentPath = window.history.state
        ? window.history.state.path
        : 'monitoring';
      if (validPaths.indexOf(currentPath) !== -1) {
        LS.sI(
          NormalizerService.generateLocalStorageKey('savePath'),
          currentPath
        );
      } else {
        LS.sI(
          NormalizerService.generateLocalStorageKey('savePath'),
          defaulpath
        );
      }
    });
    window.unload();
  }

  function isConfigurationSaved() {
    return $('.show-save-message').hasClass('not-saved');
  }

  function hideConfigurationNotSaved() {
    $('.show-save-message')
      .removeClass('not-saved')
      .hide();
  }

  function scrollTopInLowerResolutions() {
    // On click on menu scroll to top.
    $('html').on('click', 'body.smallscreen .open-left', function () {
      // We have to scroll to top only when menu is getting opened.
      if (!$('.enlarged').length) {
        $('html,body').animate({
          scrollTop: '0px'
        },
          'slow'
        );
      }
    });
  }

  function showSweetAlertError(title, text, btnConfirmText, callback) {
    // User has to set a enable shutdown in application file to enable this functionality.
    // If enable shutdown is not set then we should disable confirm button and inform user to set enable shutdown.
    var settingsDetails = NormalizerService.settingsDetails(serverData);
    var errorText = text;
    var closeAlertOnConfirm = true;
    var btnConfirmClasses = 'btn-danger';
    if (!settingsDetails.enableStopPTBtn) {
      btnConfirmClasses = 'btn-danger disabled btn-sweet-alert-confirm';
      errorText = i18next.t('alertMsgs.noEnableShutdown');
      closeAlertOnConfirm = false;
    }
    Swal.fire({
      title: title,
      text: errorText,
      type: 'warning',
      showCancelButton: true,
      cancelButtonText: i18next.t('alertMsgs.cancelBtn'),
      confirmButtonColor: '#DD6B55',
      confirmButtonText: btnConfirmText,
      reverseButtons: true,
    }).then(result => {
      if(result.value) {
          callback();
      } else {
        Swal.fire(i18next.t('alertMsgs.cancelled'));
      }
    });
  }

  function callStopAPI() {
    // We should stop the application only when password is set.
    // We are disabling button when password is not set.
    if ($('.btn-sweet-alert-confirm').hasClass('disabled')) {
      return;
    }
    // If a user clicks on "Stop" button in dialog then call "stop" API.
    Service.stopProfitTrailer().done(function (data, textStatus, jqXHR) {
      if (jqXHR.status !== PBConstants.SUCCESS_CODE) {
        toastr.info(i18next.t('constantMsgs.ajaxError'));
      }
    });
  }

  function callPublishAPI() {
    if ($('.btn-sweet-alert-confirm').hasClass('disabled')) {
      return;
    }
    Service.publishActiveSetting().done(function (data, textStatus, jqXHR) {
      if (jqXHR.status !== PBConstants.SUCCESS_CODE) {
        toastr.info(i18next.t('constantMsgs.ajaxError'));
      }
    });
  }

  function getConfigFileFromArray(file) {
    var contentString = '';
    for (var i = 0; i < file.length; i++) {
      if (file[i].startsWith('#')) {
        file[i] = 'comment-' + i + ' = ' + file[i];
      }
      contentString = contentString + file[i] + '\n';
    }
    return contentString;
  }

  function clickEventHandlers() {
    // On click on stop button show alert.
    $stopBtn.click(function () {
      if ($(this).hasClass('disabled')) {
        return;
      }
      $stopBtn.blur();
      showSweetAlertError(
        i18next.t('alertMsgs.stop'),
        i18next.t('alertMsgs.stopAlert'),
        i18next.t('alertMsgs.stopBtn'),
        callStopAPI
      );
    });

    $(document).on('click', '#loadPublicConfig', function(e) {
        e.preventDefault();
      e.stopPropagation();
      var selectedConfig = $('#publicConfigSwitcher').length
        ? $('#publicConfigSwitcher').val()
        : '';
      const newConfigName = $('input[name=publicConfigName]').length
        ? $('input[name=publicConfigName]').val()
        : '';
      $('div#dvPageContent div.message-box').hide();
      $('div#bodyLoadingSymbol').show();

      // Load all files for given public configuration.
      var pairs = Service.getConfigFile('PAIRS', selectedConfig).done(function(
        data
      ) {
        return data;
      });
      var dca = Service.getConfigFile('DCA', selectedConfig).done(function(
        data
      ) {
        return data;
      });
      var indicator = Service.getConfigFile('INDICATORS', selectedConfig).done(
        function(data) {
          return data;
        }
      );

      let promiseArray = [pairs, dca, indicator];

      Promise.all(promiseArray).then(() => {
        var pairsContent = getConfigFileFromArray(
          JSON.parse(pairs.responseText)
        );
        var dcaContent = getConfigFileFromArray(JSON.parse(dca.responseText));
        var indicatorContent = getConfigFileFromArray(
          JSON.parse(indicator.responseText)
        );

        Service.saveAll(
          newConfigName,
          pairsContent,
          dcaContent,
          indicatorContent
        ).then(() => {
          addOptionsAndSwitchConfig(newConfigName);
        });
      });
    });

    function addOption(selector, nameValue) {
      const nodeList = d.qSA(selector);
      return nodeList.forEach(function(data) {
        let optEl = document.createElement('option');
        optEl.innerHTML = nameValue;
        optEl.value = nameValue;
        data.appendChild(optEl);
      });
    }

    function addOptionsAndSwitchConfig(newConfigName) {
      if (newConfigName) {
        var newConfig = {
          configName: newConfigName,
          editingConfig: 'PAIRS',
          isUsable: true
        };
        addOption('#switchActiveConfig', newConfigName);
        addOption('#configDropdown2', newConfigName);
        $('div#bodyLoadingSymbol').hide();
        LS.sO(
          NormalizerService.generateLocalStorageKey('newConfig'),
          newConfig
        );
        $(document).trigger('evt.load-page-data');
        page('other-configs');
      }
    }

    // Highlight current page link on click on link in top menu.
    $topNav.find('li').click(function () {
      $topNav.find('li').removeClass('active');
      $(this).addClass('active');
    });

    // Highlight sub menu link on click on it.
    $topNav.find('.submenu li').click(function (e) {
      e.stopPropagation();
      $topNav.find('.submenu li').removeClass('active');
      $(this).addClass('active');
      $(this)
        .parents('.has-submenu')
        .addClass('active');
    });

    // Route to particular page based on the clicked tab.
    $(document).on('click','#sidebar-menu li, .main-heading, #topNav li, .news-dropdown-link > div,#btnChngePswrdContainer > a, #btnChangeConfigContainer , a#accountLink', function(e) {
        if ($('div.arrow,div.tooltip-inner').remove()) {
          $('div.arrow,div.tooltip-inner').remove();
        }
        // Check whether a user has unsaved changes and ask for confirmation.
        var resultsSaved = true;
        e.preventDefault();
        if (isConfigurationSaved()) {
          resultsSaved = confirm(i18next.t('constantMsgs.configNotSave'));
        }
        // If a user clicks on okay then switch to other tab
        // Otherwise stay in same tab.
        if (!resultsSaved) {
          return;
        }
        hideConfigurationNotSaved();
        removeHighlightRecordsCountInSideMenu($(this));
        var href = $(this)
          .find('a')
          .attr('href');
        if (typeof href === 'undefined') {
          href = $(this).attr('href');
        }
        page(href);
      }
    );

    // Route to particular page based on the clicked tab.
    $('#sidebar-menu li, .main-heading, #topNav li, #topConfigMenuDesktop .dropdown-content a').click(function (e) {
      // Check whether a user has unsaved changes and ask for confirmation.
      $('div#bodyLoadingSymbol').hide();
      var resultsSaved = true;
      e.preventDefault();
      if (isConfigurationSaved()) {
        resultsSaved = confirm(i18next.t('constantMsgs.configNotSave'));
      }
      // If a user clicks on okay then switch to other tab
      // Otherwise stay in same tab.
      if (!resultsSaved) {
        return;
      }
      hideConfigurationNotSaved();
      removeHighlightRecordsCountInSideMenu($(this));
      var href = $(this)
        .find('a')
        .attr('href');
        if (typeof href === 'undefined') {
          href = $(this).attr('href');
        }
      page(href);
    });
  }

  /**
   * Show loading symbol when we are calling API.
   */
  function ajaxStartCb() {
    $lastUpdatedOn.hide();
    $spinner.show();
    // To disable buy/sell button during API call.
    $('.buy-or-sell-btn').addClass('disabled');
  }

  /**
   * Show last updated on after calling API.
   */
  function ajaxStopCb() {
    $spinner.hide();
    // Show date and time.
    $monitorSummary.show();
    $lastUpdatedOn.show();
    // To enable buy/sell button after API call.
    $('.buy-or-sell-btn').removeClass('disabled');
  }

  /**
   * Get data from the server
   * @param {any} cbMain - Callback method to be called after data is fetch.
   */
  var runOnce = false;
  function getServerData(cbMain) {
    ajaxStartCb();
    $.when(Service.getMarketTrend())
      .done(function (marketTrend) {
        if (LS.gO(NormalizerService.generateLocalStorageKey('settings'))) {
          let obj = LS.gO(NormalizerService.generateLocalStorageKey('settings'));
          obj['market'] = miscLog.market;
          obj['exchange'] = miscLog.exchange || accountLog.EXCHANGE;
          obj['availableConfigs'] = JSON.parse(propertiesLog.availableConfigs);
          obj['maxConfigsAllowed'] = JSON.parse(propertiesLog.maxConfigsAllowed);
          obj['configName'] = propertiesLog.activeConfig;
          obj['ptnSettings'] = accountLog;
          LS.sO(NormalizerService.generateLocalStorageKey('settings'), obj);
        }
        disableAccountsByRole(propertiesLog.role);

        // Storing shorting boolean in local storage to use it in dataTableHelpers.js.
        localStorage.setItem(NormalizerService.generateLocalStorageKey('shorting'), JSON.stringify(propertiesLog.shorting));
        var exchange = miscLog.exchange || accountLog.EXCHANGE;
        exchange = exchange ? exchange : PBConstants.BASE_EXCHANGE;
        Array.prototype.forEach.call(d.qSA('.' + exchange.toLowerCase()), function(xels) {
          xels.style.display = 'block';
        });
        if (!runOnce) {
          runOnce = true;
          createValidPairsList(miscLog.market, marketTrend);

        }
        var responseData = cbServerData(marketTrend);
        updateShortingStatus(propertiesLog);
        if (responseData.processStatus === false) {
          toastr.info(i18next.t('constantMsgs.ajaxError'));
        }
        serverData = NormalizerService.removeUnusedData(responseData);
        Settings.setTheme(false, serverData);
        var normalizedServerData = getCurrencies();
        if (normalizedServerData) {
          cbMain(normalizedServerData);
        } else {
          serverData.currency = PBConstants.BASE_CURRENCY;
          serverData.currencyName = i18next.t(PBConstants.BASE_CURRENCY_NAME);
          serverData.currencySymbol = PBConstants.BASE_CURRENCY_SYMBOL;
          serverData.currencyValue = PBConstants.BASE_CURRENCY_VALUE;
          cbMain(serverData);
        }
      })
      .fail(function (jqXHR) {
          if (typeof jqXHR.responseText !== 'undefined' && (jqXHR.responseText.includes('ProfitTrailer Login') || jqXHR.responseText.includes('ProfitTrailer Setup'))) {
              window.location.href = PBConstants.BASE_API_URL + '/login';
          } else if (jqXHR.status === 0) {
              Settings.setTheme();
              DomHelper.toggleBtns(false);
              toastr.info(i18next.t('constantMsgs.ajaxErrorBotNotRunning'));
          } else {
              Settings.setTheme();
              DomHelper.toggleBtns(false);
              toastr.info(i18next.t('constantMsgs.ajaxError'));
          }
      })
      .always(function () {
        // lets now set a timer again for refresh
        // this will be set even if there is an error.
        // To clear timeout everytime we click on buy/sell button in datatable.
        clearTimeout(serverDataTimeout);
        serverDataTimeout = setTimeout(loadPage, PBConstants.REFRESH_TIMER);
      });
  }

  function disableAccountsByRole(role) {
    if (role.toUpperCase() === PBConstants.ENABLE_SETTINGS_BTNS_ROLE){
      $('#accountLink').addClass('hide');
      $('#switchActiveConfig').attr('disabled','true').removeClass('text-primary');
      $('#btnResetContainer i').addClass('disabled');
    }
  }

  function createValidPairsList(market, pairslist) {
    d.qSA('label[for^=price_]').forEach(function (x) {
      var y = x.innerHTML;
      var ptm = market;
      x.innerHTML = ptm + ' ' + y.split(' ').slice(1).join(' ');
    });
    let pairs = [];
    Object.keys(pairslist).forEach(function (x) {
      if (x.includes(market)) {
        pairs.push(
          x.replace(market, '').replace(/[^A-Z0-9]+/ig, '')
        );
      }
    });
    pairs.sort();
    LS.sO(NormalizerService.generateLocalStorageKey('validPairs'), pairs);
  }

  function cbServerData(marketTrend) {
    var responseData = {};
    $.extend(responseData, miscLog);
    responseData.notifications = notificationsLog;
    responseData.bbBuyLogData = possibleBuyLog;
    responseData.buyLogData = buyLog;
    responseData.dcaLogData = dcaLog;
    responseData.dustLogData = dustLog;
    responseData.accountsLog = accountLog;
    responseData.gainLogData = pairsLog;
    responseData.heartbeats = heartbeats;
    responseData.pendingLogData = pendingLog;
    responseData.stats = statsLog.basic;
    responseData.proStats = statsLog.extra;
    responseData.sellLogData = salesLog;
    responseData.watchModeLogData = watchModeLog;
    responseData.settings = propertiesLog;
    responseData.processStatus = true;
    responseData.marketTrendAvg = NormalizerService.marketTrend(
      marketTrend,
      responseData.market
    );
    if (responseData.marketTrendAvg === false) {
      responseData.processStatus = false;
      responseData.marketTrendAvg = 0;
    }
    return responseData;
  }

  function showCurrentPage() {
    if (LS.gI(NormalizerService.generateLocalStorageKey('savePath'))) {

      return page(LS.gI(NormalizerService.generateLocalStorageKey('savePath')));

    } else


      return showMonitoring();

  }


  /**
   * It redirects to monitoring page.
   */
  function showMonitoring() {
    page('monitoring');
    $('#defaultPage a').addClass('subdrop');
  }

  function intializeTooltips() {
    $('[data-toggle="tooltip"]')
      .tooltip()
      .tooltip('dispose')
      .tooltip();
  }

  /**
   * Callback function called after the data has been retrieved.
   * @param {any} data
   */
  function updateLayoutItems(data) {
    // Load current page table data.
    try {
      cbUpdateNavbar(data);
      cbUpdateFooter(data);
    } catch (e) {
      console.error(e);
      // show a message to the user.
      toastr.error(i18next.t('constantMsgs.processingError'));
    }
  }

  /** ******************************
   * START of page load callbacks
   *******************************/
  function cbPairLog() {
    let sumVals = TABLE_REFS.loadPairLogsData(pairsLog, watchModeLog);
    let percentages = [];
    let sum, avg = 0;

    if (propertiesLog.hasOwnProperty('isLeverageExchange') && propertiesLog.isLeverageExchange) {
      pairsLog.concat(watchModeLog).forEach(pair => {
        if(propertiesLog.shorting) {
          percentages.push(((((pair.avgPrice / pair.currentPrice) - 1) * 100) - pair.fee) * (pair.leverage > 0 ? pair.leverage : 1));
        } else {
          percentages.push(((((pair.currentPrice / pair.avgPrice) - 1) * 100) - pair.fee) * (pair.leverage > 0 ? pair.leverage : 1));
        }
      });

      if (percentages.length > 0) {
        sum = percentages.reduce((previous, current) => current += previous);
        avg = sum / percentages.length;
      }
      if(propertiesLog.shorting) {
        addDifference(sumVals.totalCost, sumVals.currentValue, 'pairsLogDifference', true, avg, true);
      } else {
        addDifference(sumVals.currentValue, sumVals.totalCost, 'pairsLogDifference', true, avg, true);
      }

    } else {
      addDifference(sumVals.currentValue, sumVals.totalCost, 'pairsLogDifference', true);
    }
    $('#pairsLogTotalCurrentVal').html(sumVals.currentValueConvertedValue);
    $('#pairsLogRealCost').html(sumVals.totalCostConvertedValue);
  }

  /**
   * Getters for api/v2/ routes
   */
  function getPossibleBuyLog() {
    $.when(Service.getPossibleBuyLog()).done(function(data) {
      setPossibleBuyLog(data);
    });
  }
  function getPairsLog() {
    $.when(Service.getPairsLog()).done(function(data) {
      setPairsLog(data);
    });
  }
  function getWatchModeLog() {
    $.when(Service.getWatchModeLog()).done(function(data) {
      setWatchModeLog(data);
    });
  }
  function getDcaLog() {
    $.when(Service.getDcaLog()).done(function(data) {
      setDcaLog(data);
    });
  }
  function getPendingLog() {
    $.when(Service.getPendingLog()).done(function(data) {
      setPendingLog(data);
    });
  }
  function getBuyLog() {
    $.when(Service.getBuyLog()).done(function(data) {
      setBuyLog(data);
    });
  }
  function getSalesLog() {
    $.when(Service.getSalesLog()).done(function(data) {
      setSalesLog(data);
    });
  }
  function getDustLog() {
    $.when(Service.getDustLog()).done(function(data) {
      setDustLog(data);
    });
  }
  function getReservedCoins() {
    $.when(Service.getReservedCoins()).done(function(data) {
      setReservedCoins(data);
    });
  }
  function getReversalCoins() {
    $.when(Service.getReversalCoins()).done(function(data) {
      setReversalCoins(data);
    });
  }

  function getStatsLog() {
    $.when(Service.getStatsLog()).done(function(data) {
      setStatsLog(data);
    });
  }
  function getPropertiesLog() {
    $.when(Service.getPropertiesLog()).done(function(data) {
      setPropertiesLog(data);
    });
  }
  function getNotificationsLog() {
    $.when(Service.getNotificationsLog()).done(function(data) {
      setNotificationsLog(data);
    });
  }
  function getHeartbeatsLog() {
    $.when(Service.getHeartbeatsLog()).done(function(data) {
      setHeartbeatsLog(data);
    });
  }
  function getMiscLog() {
    $.when(Service.getMiscLog()).done(function(data) {
      setMiscLog(data);
    });
  }
  function getAccountLog() {
    $.when(Service.getAccountLog()).done(function(data) {
      setAccountLog(data);
    });
  }

  /**
   * Setters for api/v2/ routes
   * @param data
   */
  function setPossibleBuyLog(data) {
    possibleBuyLog = data;
  }
  function setPairsLog(data) {
    pairsLog = data;
  }
  function setWatchModeLog(data) {
    watchModeLog = data;
  }
  function setDcaLog(data) {
    dcaLog = data;
  }
  function setPendingLog(data) {
    pendingLog = data;
  }
  function setBuyLog(data) {
    buyLog = data;
  }
  function setSalesLog(data) {
    salesLog = data;
  }
  function setDustLog(data) {
    dustLog = data;
  }
  function setReservedCoins(data) {
    reservedCoins = data;
  }
  function setReversalCoins(data) {
    reversalCoins = data;
  }

  function setStatsLog(data) {
    statsLog = data;
  }
  function setPropertiesLog(data) {
    propertiesLog = data;
  }
  function setNotificationsLog(data) {
    notificationsLog = data;
  }
  function setHeartbeatsLog(data) {
    heartbeats = data;
  }
  function setMiscLog(data) {
    miscLog = data;
  }
  function setAccountLog(data) {
    accountLog = data;
  }


  function cbPossibleBuys() {
    TABLE_REFS.loadPossibleBuyData(possibleBuyLog);
  }

  function cbBuyLog() {
    TABLE_REFS.loadBuyLogData(buyLog.data)
  }

  function cbDustLog() {
    // Dust Log
    var sumValsDust = TABLE_REFS.loadDustLogsData(dustLog);
    addDifference(
        sumValsDust.currentValue,
        sumValsDust.totalCost,
      'dustLogDifference',
      true
    );
    $('#dustLogTotalCurrentVal').html(sumValsDust.currentValueConvertedValue);
    $('#dustLogRealCost').html(sumValsDust.totalCostConvertedValue);

    // Reserved Logs
    TABLE_REFS.loadReservedLogsData(reservedCoins);

    // Reversal Logs
    TABLE_REFS.loadReversalLogsData(reversalCoins);
  }

  function cbDcaLog() {
    let sumVals = TABLE_REFS.loadDcaLogsData(dcaLog);
    let percentages = [];
    let sum, avg = 0;

    if (propertiesLog.hasOwnProperty('isLeverageExchange') && propertiesLog.isLeverageExchange) {
      dcaLog.forEach(pair => {
        if(propertiesLog.shorting) {
          percentages.push(((((pair.avgPrice / pair.currentPrice) - 1) * 100) - pair.fee) * (pair.leverage > 0 ? pair.leverage : 1));
        } else {
          percentages.push(((((pair.currentPrice / pair.avgPrice) - 1) * 100) - pair.fee) * (pair.leverage > 0 ? pair.leverage : 1));
        }
      });

      if(percentages.length > 0) {
        sum = percentages.reduce((previous, current) => current += previous);
        avg = sum / percentages.length;
      }

      if(propertiesLog.shorting) {
        addDifference(sumVals.totalCost, sumVals.currentValue, 'dcLogDifference', true, avg, true)
      } else {
        addDifference(sumVals.currentValue, sumVals.totalCost, 'dcLogDifference', true, avg, true)
      }
    } else {
      addDifference(sumVals.currentValue, sumVals.totalCost, 'dcLogDifference', true);
    }

    $('#dcLogTotalCurrentVal').html(sumVals.currentValueConvertedValue);
    $('#dcLogRealCost').html(sumVals.totalCostConvertedValue);
  }

  function cbPendingLog() {
    var sumVals = TABLE_REFS.loadPendingLogsData(pendingLog);
    addDifference(
      sumVals.currentValue,
      sumVals.totalCost,
      'pendingLogDifference',
      true
    );
    $('#pendingLogTotalCurrentVal').html(sumVals.currentValueConvertedValue);
    $('#pendingLogRealCost').html(sumVals.totalCostConvertedValue);
  }

  function cbNews() {
    AppFeed.displayNewsPageNews();
  }

  function cbSalesLog() {
    var sumVals = TABLE_REFS.loadSellLogData(salesLog.data);
    let percentages = [];
    let sum, avg = 0;

    if (propertiesLog.hasOwnProperty('isLeverageExchange') && propertiesLog.isLeverageExchange) {
      salesLog.data.forEach(pair => {
        if(propertiesLog.shorting) {
          percentages.push(((((pair.totalCost / pair.soldValue) - 1) * 100) - pair.fee) * (pair.leverage > 0 ? pair.leverage : 1));
        } else {
          percentages.push(((((pair.soldValue / pair.totalCost) - 1) * 100) - pair.fee) * (pair.leverage > 0 ? pair.leverage : 1));
        }
      });

      if (percentages.length > 0) {
        sum = percentages.reduce((previous, current) => current += previous);
        avg = sum / percentages.length;
      }

      if(propertiesLog.shorting) {
        addDifference(sumVals.boughtCost, sumVals.soldValue, 'salesLogDifference', true, avg, true);
      } else {
        addDifference(sumVals.soldValue, sumVals.boughtCost, 'salesLogDifference', true, avg, true);
      }

    } else {
      addDifference(sumVals.soldValue, sumVals.boughtCost, 'salesLogDifference', true);
    }


    $('#salesLogTotalCurrentVal').html(sumVals.soldValueConvertedValue);
    $('#salesLogBoughtCost').html(sumVals.boughtCostConvertedValue);
  }

  /**
   * It adds difference value and percentage in the summary table.
   */
  function addDifference(firstValue, secondValue, element, addCurrencyValue, percentageAverageFromShorting = 0, isShorting = false) {
    let difference = firstValue - secondValue;
    let diffPerTmp;

    if(isShorting) {
      diffPerTmp = percentageAverageFromShorting;
    } else {
      diffPerTmp = +secondValue ? (difference / secondValue) * 100 : 0;
    }

    let diffValue = DataTableHelper.normalizeValueWithCommas(
        difference,
        'display',
        '',
        '',
        '',
        serverData.currentMarketPrecision
    );
    let currentCurrencyValue = '';
    if (addCurrencyValue) {
      currentCurrencyValue = NormalizerService.getInCurrencyValue(diffValue, serverData, 2, true);
      currentCurrencyValue = currentCurrencyValue ? '<span class="current-currency-values"> (' + serverData.currencySymbol + currentCurrencyValue + ')</span>' : '';
    }

    let differencePerClass = diffPerTmp < 0 ? PBConstants.NEGATIVE_CLASS_TEXT : PBConstants.POSITIVE_CLASS_TEXT;
    let differencePer = ' (' + parseFloat(diffPerTmp.toFixed(2)).toFixed(2) + ' %)';
    $('#' + element + ' .value').html(diffValue + currentCurrencyValue);
    $('#' + element + ' .percentage').html(differencePer).removeClass(PBConstants.POSSIBLE_CLASSES).addClass(differencePerClass);
  }

  /**
   * We are having tooltips for child and parent elment.
   * Below code is to fix two tooltips are appearing at a time when we hover
   * on child element.
   */
  function handleDustValueTooltips() {
    $('.js-dust-values').hover(function () {
      $(this)
        .parents('.card-box')
        .tooltip('hide');
    });
  }

  function cbAccount() {
    initializeAccount(accountLog, propertiesLog);
    cbAPILogs();
  }

  function cbHotConfig() {
    if(!propertiesLog.hasOwnProperty('testMode')) {
      return;
    }
    if(propertiesLog.testMode || propertiesLog.testnet) {
      document.getElementById('HCBalanceParent').classList.remove('hide');
    }
    let dcaLevels = [], boughtPrice = [];

    dcaLog.forEach(element => {
      dcaLevels.push(element.currency);
    });

    pairsLog.concat(watchModeLog).forEach(element => {
      boughtPrice.push(element.currency);
    });

    dcaLevels = dcaLevels.sort();
    boughtPrice = boughtPrice.concat(dcaLevels).sort();

    dcaLevels.forEach(element => {
      let selectOption = document.createElement('option');
      selectOption.text = element;
      selectOption.value = element;

      let selectElement = document.getElementById('HCSetBuyTimesPairs');
      selectElement.appendChild(selectOption);
    });

    boughtPrice.forEach(element => {
      let selectOption = document.createElement('option');
      selectOption.text = element;
      selectOption.value = element;

      let selectElement = document.getElementById('HCBoughtPricePairs');
      selectElement.appendChild(selectOption);
    });
  }

  function cbStats() {
    initializeStats(statsLog, propertiesLog, salesLog);
  }

  function cbMonitoring(data) {
    intializeTooltips();
    var summary = NormalizerService.getMonitoringSummary(data);

    var tablesInfo = getTablesInfo(data);
    if (!summary || !tablesInfo) {
      return;
    }
    var normalizedSummary = NormalizerService.normalizeSummaryData(summary);

    // Top market values
    if (data.hasOwnProperty('coinTopMarket')) {
      CoinMarket.bindTopMarketValues(summary, data);
    }

    // Global market values
    if (data.hasOwnProperty('coinGlobalMarket')) {
      CoinMarket.bindGlobalMarketValues(summary, data);
    }

    // Tables summary
    // DCA log table summary.
    var dcLogsData = tablesInfo['dca-log'];
    let percentagesDCA = [];
    let sumDCA, avgDCA = 0;

    jQuery('#mDcLogCurrentValue').html(dcLogsData.currentValueConvertedValue);
    jQuery('#mDcLogBoughtCost').html(dcLogsData.boughtCostConvertedValue);

    if (propertiesLog.hasOwnProperty('isLeverageExchange') && propertiesLog.isLeverageExchange) {
      dcaLog.forEach(pair => {
        if(propertiesLog.shorting) {
          percentagesDCA.push(((((pair.avgPrice / pair.currentPrice) - 1) * 100) - pair.fee) * (pair.leverage > 0 ? pair.leverage : 1));
        } else {
          percentagesDCA.push(((((pair.currentPrice / pair.avgPrice) - 1) * 100) - pair.fee) * (pair.leverage > 0 ? pair.leverage : 1));
        }
      });

      if(percentagesDCA.length > 0) {
        sumDCA = percentagesDCA.reduce((previous, current) => current += previous);
        avgDCA = sumDCA / percentagesDCA.length;
      }
      if(propertiesLog.shorting) {
        addDifference(dcLogsData.boughtCost, dcLogsData.currentValue, 'mDcLogDifference', false, avgDCA, true)
      } else {
        addDifference(dcLogsData.currentValue, dcLogsData.boughtCost, 'mDcLogDifference', false, avgDCA, true)
      }
    } else {
      addDifference(dcLogsData.currentValue, dcLogsData.boughtCost, 'mDcLogDifference');
    }

    // Pairs log table summary.
    var pairsLogData = tablesInfo['pairs-log'];
    let percentagesPairs = [];
    let sumPairs, avgPairs = 0;

    jQuery('#mPairsLogCurrentValue').html(
      pairsLogData.currentValueConvertedValue
    );
    jQuery('#mPairsLogBoughtCost').html(pairsLogData.boughtCostConvertedValue);

    if (propertiesLog.hasOwnProperty('isLeverageExchange') && propertiesLog.isLeverageExchange) {
      pairsLog.concat(watchModeLog).forEach(pair => {
        if(propertiesLog.shorting) {
          percentagesPairs.push(((((pair.avgPrice / pair.currentPrice) - 1) * 100) - pair.fee) * (pair.leverage > 0 ? pair.leverage : 1));
        } else {
          percentagesPairs.push(((((pair.currentPrice / pair.avgPrice) - 1) * 100) - pair.fee) * (pair.leverage > 0 ? pair.leverage : 1));
        }
      });

      if (percentagesPairs.length > 0) {
        sumPairs = percentagesPairs.reduce((previous, current) => current += previous);
        avgPairs = sumPairs / percentagesPairs.length;
      }
      if(propertiesLog.shorting) {
        addDifference(pairsLogData.boughtCost, pairsLogData.currentValue, 'mPairsLogDifference', false, avgPairs, true);
      } else {
        addDifference(pairsLogData.currentValue, pairsLogData.boughtCost, 'mPairsLogDifference', false, avgPairs, true);
      }
    } else {
      addDifference(pairsLogData.currentValue, pairsLogData.boughtCost, 'mPairsLogDifference');
    }


    // Pending log table summary.
    var pendingLogData = tablesInfo['pending-log'];
    jQuery('#mPendingLogCurrentValue').html(
      pendingLogData.currentValueConvertedValue
    );
    jQuery('#mPendingLogBoughtCost').html(
      pendingLogData.boughtCostConvertedValue
    );
    addDifference(
      pendingLogData.currentValue,
      pendingLogData.boughtCost,
      'mPendingLogDifference'
    );

    // Sales log table summary
    var salesLogData = tablesInfo['sales-log'];
    let percentagesSales = [];
    let sumSales, avgSales = 0;

    jQuery('#mSalesLogCurrentValue').html(salesLogData.currentValueConvertedValue);
    jQuery('#mSalesLogBoughtCost').html(salesLogData.boughtCostConvertedValue);

    if (propertiesLog.hasOwnProperty('isLeverageExchange') && propertiesLog.isLeverageExchange) {
      salesLog.data.forEach(pair => {
        if(propertiesLog.shorting) {
          percentagesSales.push(((((pair.totalCost / pair.soldValue) - 1) * 100) - pair.fee) * (pair.leverage > 0 ? pair.leverage : 1));
        } else {
          percentagesSales.push(((((pair.soldValue / pair.totalCost) - 1) * 100) - pair.fee) * (pair.leverage > 0 ? pair.leverage : 1));
        }
      });

      if (percentagesSales.length > 0) {
        sumSales = percentagesSales.reduce((previous, current) => current += previous);
        avgSales = sumSales / percentagesSales.length;
      }

      if(propertiesLog.shorting) {
        addDifference(salesLogData.boughtCost, salesLogData.currentValue, 'mSalesLogDifference', false, avgSales, true);
      } else {
        addDifference(salesLogData.currentValue, salesLogData.boughtCost, 'mSalesLogDifference', false, avgSales, true);
      }
    } else {
      addDifference(salesLogData.currentValue, salesLogData.boughtCost, 'mSalesLogDifference', false);
    }

    var firstRowElementsObjObj = {
      balance: ['mBalanceVal', '', 'balance'],
      DCABalance: ['mDCABalanceVal', '', 'DCABalance'],
      pairsBalance: ['mPairsBalanceVal', '', 'pairsBalance'],
      reversalReservedBalance: ['mReversalBalanceVal', '', 'reversalReservedBalance'],
      totalCurrentVal: ['mTotalCurrentVal', '', 'totalCurrentVal'],
      startBalance: ['mStartBalance', '', 'startBalance'],
      lastWeekProfit: ['mLastWeekProfit', '', 'lastWeekProfit'],
      lastWeekProfitSales: ['mLastWeekProfitSales', '', 'lastWeekProfitSales'],
      yesterdayProfit: ['mYesterdayProfit', '', 'yesterdayProfit'],
      lastWeekMarketVolume: [
        'mLastWeekMarketVolume',
        '',
        'lastWeekMarketVolume'
      ],
      yesterdayMarketVolume: [
        'mYesterdayMarketVolume',
        '',
        'yesterdayMarketVolume'
      ],
      todayMarketVolume: ['mTodayMarketVolume', '', 'todayMarketVolume'],
      yesterdayProfitSales: [
        'mYesterdayProfitSales',
        '',
        'yesterdayProfitSales'
      ],
      todayProfit: ['mTodayProfit', '', 'todayProfit'],
      todayProfitSales: ['mTodayProfitSales', '', 'todayProfitSales'],
      balMarketPrice: ['mBalUSDValue'],
      TCVMarketPrice: ['mTCVUSDValue'],
      TCVDustValue: ['mTCVDustValue'],
      TPVMarketPrice: ['mTPVUSDValue'],
      lastWeekProfitUSDValue: [
        'mLastWeekProfitUSDValue',
        '',
        'lastWeekProfitUSDValue'
      ],
      yesterdayProfitUSDValue: [
        'mYesterdayProfitUSDValue',
        '',
        'yesterdayProfitUSDValue'
      ],
      yesterdayProfitPercent: [
        'mYesterdayProfitPercent',
        '',
        'yesterdayProfitPercent'
      ],
      lastWeekProfitPercent: ['mLastWeekProfitPercent', '', 'lastWeekProfitPercent'],
      todayProfitUSDValue: ['mTodayProfitUSDValue', '', 'todayProfitUSDValue'],
      todayProfitPercent: ['mTodayProfitPercent', '', 'todayProfitPercent']
    };
    DomHelper.checkAndBindData(firstRowElementsObjObj, normalizedSummary);

    var secondRowElementsObj = {
      percChange1h: ['mTrend1h', 'percChange1hClass'],
      percChange24h: ['mTrend24h', 'percChange24hClass'],
      percChange7d: ['mTrend7d', 'percChange7dClass'],
      marketTrendAvg: ['mMarketTrend', 'marketTrendAvgClass', 'marketTrendAvg'],
      priceUsd: ['mBtcPrice'],
    };
    DomHelper.checkAndBindData(
      secondRowElementsObj,
      summary,
      PBConstants.POSSIBLE_CLASSES,
      summary
    );

    var monitoringSOMElements = {
      sellOnlyMode: ['mSellOnlyMode', 'sellOnlyModeClass'],
    };
    DomHelper.checkAndBindData(
      monitoringSOMElements,
      summary,
      PBConstants.POSSIBLE_BADGES,
      summary
    );
    $('#mMarketTrendLabel').html(
      summary.apiMarket + ' ' + i18next.t('monitoringSection.trend24H')
    );
    $('#mMarketTrendContainer').attr(
      'data-original-title',
      i18next.t('monitoringSection.trendContainerFirstPart') +
      ' ' +
      summary.apiMarket +
      ' ' +
      i18next.t('monitoringSection.trendContainerSecondPart')
    );
    jQuery('.market').html(summary.apiMarket);
    $('.js-exchange').html(data.exchange);
    jQuery('#mMarket').html(summary.apiMarket);
    jQuery('#mVersion').html(data.version);
    jQuery('#mBtc24h')
      .html('(' + summary.percBtcChange24h + ')')
      .removeClass(PBConstants.POSSIBLE_CLASSES)
      .addClass(summary.percChange24hBtcClass);
    jQuery('#mSellOnlyModeLabel').attr(
      'data-original-title',
      summary.sellOnlyModeToolTip
    );
    jQuery('.js-usd-value .full-text').html(summary.estimatedValueCurrencyText);
    jQuery('.js-usd-value .short-text').html(summary.estimatedValueShortText);
    changeMarketIcons(summary.apiMarket);
    var $marketPriceCalculations = $('.js-usd-value').parents(
      '.market-price-calculations'
    );
    if (
      summary.apiMarket === PBConstants.HIDE_PROFIT_MARKET_AND_ESTIMATED_USD &&
      data.currency === PBConstants.BASE_CURRENCY
    ) {
      $marketPriceCalculations.hide();
    } else {
      $marketPriceCalculations.show();
    }
  }

  /**
   * Removes all the previously added market icons and adds current market icon in monitoring first row.
   * @param {*} market
   */
  function changeMarketIcons(market) {
    var $marketIconElement = $('.market-icon');
    // Generate all market icons array.
    var possibleMarketIconsArr = $.map(marketIcons, function (icon) {
      return [icon];
    });
    var currentMarketIcon = marketIcons.hasOwnProperty(market) ?
      marketIcons[market] :
      marketIcons.BTC;
    DomHelper.replaceClasses(
      $marketIconElement,
      possibleMarketIconsArr,
      currentMarketIcon
    );
  }

  function updatePageTitle(tableRecordsCount) {
    // We have to show (possibleLogsLength-pairsLogsLength-DCALogsLength) in page title.
    var logsCount = +tableRecordsCount['possible-buys-log'].toString() +
      '-' +
      tableRecordsCount['pairs-log'].toString() +
      '-' +
      tableRecordsCount['dca-log'].toString();
    var sitename = i18next.t('constantMsgs.ptMonitor');
    if (accountLog.SITE_NAME) {
      sitename = accountLog.SITE_NAME;
    }
    DomHelper.updatePageTitle(sitename + ' ' + logsCount);
  }

  function cbUpdateNavbar(data) {
    var summaryTemp = NormalizerService.getMonitoringSummary(data);
    var tables = [
      'pairs-log',
      'pending-log',
      'dca-log',
      'possible-buys-log',
      'sales-log',
      'misc-log',
      'buy-log'
    ];
    var oldTableRecordsCount = tableRecordsCount;
    var currentPage = window.history.state ? window.history.state.path : 'monitoring';
    var navBarElements = {
      'possible-buys-log': ['possibleBuyLogLength'],
      'pairs-log': ['pairsLogLength'],
      'pending-log': ['pendingLogLength'],
      'sales-log': ['salesLogLength'],
      'dca-log': ['dcLogLength'],
      'misc-log': ['dustLogLength'],
      'buy-log': ['buyLogLength']
    };
    tableRecordsCount = NormalizerService.getTablesRecordsCount(
        routes,
        tables,
        possibleBuyLog ? possibleBuyLog.length : 0,
        pairsLog ? pairsLog.length : 0,
        dcaLog ? dcaLog.length : 0,
        pendingLog ? pendingLog.length : 0,
        buyLog.data ? buyLog.data.length : 0,
        salesLog.data ? salesLog.data.length : 0,
        dustLog ? dustLog.length : 0,
        reservedCoins ? reservedCoins.length : 0,
        reversalCoins ? reversalCoins.length : 0,
        watchModeLog ? watchModeLog.length : 0,
    );
    var topBarRecordsCountElements = {
      'possible-buys-log': ['topPossibleBuyLogLength'],
      'pairs-log': ['topPairsLogLength'],
      'pending-log': ['topPendingLogLength'],
      'sales-log': ['topSalesLogLength'],
      'dca-log': ['topDcLogLength'],
      'misc-log': ['topDustLogLength'],
      'buy-log': ['topBuyLogLength']
    };

    updatePageTitle(tableRecordsCount);
    if (!summaryTemp && !tableRecordsCount) {
      return;
    }
    // Records count in side menu
    highlightRecordsCountInSideMenu(
      oldTableRecordsCount,
      tableRecordsCount,
      currentPage,
      navBarElements
    );
    // Records count in top menu
    highlightRecordsCountInSideMenu(
      oldTableRecordsCount,
      tableRecordsCount,
      currentPage,
      topBarRecordsCountElements
    );

    var summary = NormalizerService.normalizeSummaryData(summaryTemp, 2);
    var precisionTo3 = {
      balance: 'balance',
      totalCurrentVal: 'totalCurrentVal',
      startBalance: 'startBalance',
      lastWeekProfit: 'lastWeekProfit',
      yesterdayProfit: 'yesterdayProfit',
      todayProfit: 'todayProfit'
    };
    var nrmlsdTmpData = NormalizerService.addPrecisionForMultipleValues(
      precisionTo3,
      summaryTemp,
      3
    );
    $.extend(summary, nrmlsdTmpData);
    $monitorSummary.addClass(updatingSummaryClass);
    var navBarElementsObj = {
      balance: ['nBalanceVal', '', 'balance'],
      totalCurrentVal: ['nTotalCurrentVal', '', 'totalCurrentVal'],
      startBalance: ['nStartBalance', '', 'startBalance'],
      marketTrendAvg: ['nMarketTrend', 'marketTrendAvgClass', 'marketTrendAvg'],
      normalizedMarketPrice: ['nMarketPrice', '', 'normalizedMarketPrice'],
      normalizedMarket: ['nMarket', '', 'marketTitle'],
      lastWeekProfit: ['nLastWeekProfit', '', 'lastWeekProfit'],
      yesterdayProfit: ['nYesterdayProfit', '', 'yesterdayProfit'],
      todayProfit: ['nTodayProfit', '', 'todayProfit'],
      currencySymbol: ['apiCurrency']
    };
    DomHelper.checkAndBindData(
      navBarElementsObj,
      summary,
      PBConstants.POSSIBLE_CLASSES,
      summary
    );
    jQuery('#nMarketPercChange')
      .html('(' + summary.normalizedMarketPercChange + ')')
      .attr('title', summary.normalizedMarketPercChange)
      .removeClass(PBConstants.POSSIBLE_CLASSES)
      .addClass(summary.normalizedMarketPercChangeClass);
    $('#nMarketTrendLabel').attr(
      'title',
      i18next.t('summaryItems.trend.title') +
      ' ' +
      summary.apiMarket +
      i18next.t('summaryItems.trend.suffix')
    );
    $monitorSummary.removeClass(updatingSummaryClass);
    var monitorInfoMblTxt = $($('.monitor-summary').get(0).outerHTML)
      .addClass('mointor-summary-title')
      .removeClass('monitor-summary')
      .get(0).outerHTML;
    $('.monitor-info-mbl')
      .find('a')
      .attr('data-original-title', monitorInfoMblTxt);
    $('#apiCurrency').attr('data-original-title', summary.currencySymbolTitle);
    updateSubHeader(data);


  }

  /**
   * It highlights the records count in side menu if they got changed.
   * @param {*} oldTableRecordsCount
   * @param {*} tableRecordsCount
   * @param {*} currentPage
   * @param {*} navBarElements
   */
  function highlightRecordsCountInSideMenu(
    oldTableRecordsCount,
    tableRecordsCount,
    currentPage,
    navBarElements
  ) {
    var classedObj = {};
    var recordsCountClasses = $.isEmptyObject(oldTableRecordsCount) ?
      '' :
      RECORDS_COUNT_CLASSES;
    // On page load oldTablerecordscount will be empty.
    if (!$.isEmptyObject(oldTableRecordsCount)) {
      // Should not highlight current page records count.
      delete oldTableRecordsCount[currentPage];
      // Add current page to nav bar elements to add primary badge to current page logs count.
      if (navBarElements[currentPage]) {
        classedObj[currentPage] = DEFAULT_RECORDS_COUNT_CLASS;
        navBarElements[currentPage].push(currentPage);
      }

      for (var table in oldTableRecordsCount) {
        // If previous records count and current records count are not equal then highlight that records count.
        if (oldTableRecordsCount[table] !== tableRecordsCount[table]) {
          classedObj[table] = RECORDS_COUNT_HIGHLIGHT_CLASS;
          navBarElements[table].push(table);
        } else {
          classedObj[table] = DEFAULT_RECORDS_COUNT_CLASS;
          navBarElements[table].push(table);
        }
      }
    }
    DomHelper.checkAndBindData(
      navBarElements,
      tableRecordsCount,
      recordsCountClasses,
      classedObj
    );
  }

  /**
   * When a user clicks on menu item then remove records count highlighted color in side menu
   * and show in normal color.
   */
  function removeHighlightRecordsCountInSideMenu($currentMenuItem) {
    $currentMenuItem
      .find('.records-count')
      .removeClass(RECORDS_COUNT_HIGHLIGHT_CLASS)
      .addClass(DEFAULT_RECORDS_COUNT_CLASS);
  }

  function cbUpdateFooter(data) {
    var elems = {
      version: ['apiVersion']
    };
    DomHelper.checkAndBindData(elems, data);
    jQuery('.market').html(data.market);
    $('.js-exchange').html(data.exchange);
  }

  /**
   * It updates second header.
   * @param {*} dataTmp
   */
  function updateSubHeader(dataTmp) {
    var data = $.extend(
      dataTmp,
      NormalizerService.getSecondHeaderData(dataTmp)
    );
    var elems = {
      market: ['apiMarket'],
      sellOnlyMode: ['apiSellOnlyMode', 'sellOnlyModeClass'],
      botType: ['botType']
    };
    DomHelper.checkAndBindData(elems, data, PBConstants.POSSIBLE_BADGES, data);
    $('#apiSellOnlyModeLabel').attr(
      'data-original-title',
      data.sellOnlyModeToolTip
    );
    var modeText = '';
    if(data.isTestMode){
      modeText = i18next.t('summaryItems.testMode.label');
    }
    if(data.isTestNet) {
      modeText = i18next.t('summaryItems.testNet.label');
    }
    if(data.isMargin) {
      modeText = modeText + ' ' + i18next.t('summaryItems.marginTrading.label');
    }
    $testModeContainer.hide();
    if (modeText && modeText.length > 0){
        $testModeContainer.text(modeText.trim()).show();
    }

    enableTestModeButtons();
  }

  // On window resize set ace editor height.
  function handleWindowResize() {
    var resized;
    $(window).on('resize orientationChanged', function () {
      clearTimeout(resized);
      resized = setTimeout(function () {
        setConfigurationContainerHeight();
        setMainContentMarginAndHeight();
      }, 400);
    });
  }

  /**
    * It shows API logs menu only if password is set and role is not "M"
    * @param {*} data
    */
  function showOrHideAPILogsMenu(data) {
    if (propertiesLog.passwordSet && propertiesLog.role === PBConstants.ENABLE_SETTINGS_BTNS_ROLE) {
      $APILogsMenu.show();
    } else {
      $APILogsMenu.hide();
      // If a user is in API logs page then redirect to monitoring page.
      var currentPath = window.history.state ? window.history.state.path : 'monitoring';
      if (currentPath.indexOf('api-logs') !== -1) {
        showMonitoring();
      }
    }
  }


  /**
   * It hides the configuration menu if enable config property
   * is false. It shows the configuration menu if enable config property
   * is true.
   * @param {*} data
   */
  function showOrHideConfigurationMenu() {
    if (propertiesLog.hasOwnProperty('enableConfig') && propertiesLog.enableConfig && propertiesLog.passwordSet) {
      $configurationMenu.show();
    } else {
      $configurationMenu.hide();
      // If a user is in configuration pages then redirect to monitoring page and
      // collapse the configuration menu
      var currentPath = window.history.state ? window.history.state.path : 'monitoring';
      if (currentPath.indexOf('config') !== -1) {
        $configurationMenu.find('a').trigger('click');
        // In lower resolutions
        var lowerResolutionsMenu = $('.smallscreen #configMenu a');
        if (lowerResolutionsMenu.length) {
          lowerResolutionsMenu.trigger('touchstart');
        }
        showMonitoring();
      }
    }
  }

  function cbAPILogs() {
    intializeTooltips();
    if (APILogsEditor) {
      APILogsEditor.destroy();
    }
    // Set ace editor height
    setConfigurationContainerHeight();
    APILogsEditor = DomHelper.getGlobalEditor('APILogsEditor');
    APILogs.setAPILogsContent(APILogsEditor, false);
  }

  function cbPTFeeder() {

    if(propertiesLog.isPTFeederEnabled) {
      APILogs.callPTFeederLogs();
      APILogs.callPTFeederHostConfig();
      APILogs.callPTFeederAppConfig();
    }

    intializeTooltips();
    if (PTFeederEditor) {
      PTFeederEditor.destroy();
    }
    // Set ace editor height
    setConfigurationContainerHeight();
    PTFeederEditor = DomHelper.getGlobalEditor('PTFeederLog');
    APILogs.setPTFeederContent(PTFeederEditor, false);

    // PTFeeder configuration editor for App settings
    if(PTFConfigEditorApp) {
      PTFConfigEditorApp.destroy();
    }
    // setConfigurationContainerHeight();
    PTFConfigEditorApp = DomHelper.getGlobalEditor('PTFConfigEditorApp');
    PTFConfigEditorApp.getSession().setMode('ace/mode/json');
    PTFConfigEditorApp.getSession().setTabSize(2);
    PTFConfigEditorApp.getSession().setUseWorker(false);

    APILogs.setPTFeederAppConfig(PTFConfigEditorApp);

    // PTFeeder configuration editor for Host settings
    if(PTFConfigEditorHost) {
      PTFConfigEditorHost.destroy();
    }
    // setConfigurationContainerHeight();
    PTFConfigEditorHost = DomHelper.getGlobalEditor('PTFConfigEditorHost');
    PTFConfigEditorHost.getSession().setMode('ace/mode/json');
    PTFConfigEditorHost.getSession().setTabSize(2);
    PTFConfigEditorHost.getSession().setUseWorker(false);

    APILogs.setPTFeederHostConfig(PTFConfigEditorHost);
  }

  function cbLoadConfig() {
    var currentConfigType = window.location.pathname ? window.location.pathname.replace('/', '') : 'active-config';
    LS.sO(NormalizerService.generateLocalStorageKey('settings'), {});

    var newConfig = LS.gO(NormalizerService.generateLocalStorageKey('newConfig'));


    intializeTooltips();
    if (editor) {
      editor.destroy();
    }
    var settingsObj = LS.gO(NormalizerService.generateLocalStorageKey('settings'));
    if (settingsObj == null) {
      return;
    }

    if (newConfig !== null && newConfig.isUsable === true) {
      settingsObj.editingFile = newConfig.editingConfig;
      settingsObj.editingConfig = newConfig.configName;
      settingsObj.configName = newConfig.configName;
      LS.sO(
        NormalizerService.generateLocalStorageKey('settingsObj'),
        settingsObj
      );
      newConfig.isUsable = false;
      LS.sO(NormalizerService.generateLocalStorageKey('newConfig'), newConfig);
    }

    var editingFile = settingsObj['editingFile'];
    if (currentConfigType === 'active-config') {
      settingsObj['editingConfig'] = settingsObj['configName'];
      LS.sO(NormalizerService.generateLocalStorageKey('settings'), settingsObj);
    }
    if (editingFile === undefined) {
      settingsObj['editingFile'] = 'PAIRS';
      LS.sO(NormalizerService.generateLocalStorageKey('settings'), settingsObj);
      editingFile = 'PAIRS';
    }

    var configName = settingsObj['configName'];
    if (configName === undefined && serverData.settings !== undefined) {
      settingsObj['configName'] = serverData.settings.activeConfig;
      settingsObj['editingConfig'] = serverData.settings.activeConfig;
      LS.sO(NormalizerService.generateLocalStorageKey('settings'), settingsObj);
      configName = serverData.settings.activeConfig;
    }

    document.querySelector('#switchActiveConfig').value = configName;
    // Set ace editor height
    setConfigurationContainerHeight();
    editor = DomHelper.getGlobalEditor('PBConfigEditor');
    // Add CTRL/CMD-S to save config file in Ace Editor
    editor.commands.addCommand({
      name: 'save',
      bindKey: {
        win: 'Ctrl-S',
        'mac': 'Cmd-S'
      },
      exec: function () {
        var settingsObj = LS.gO(NormalizerService.generateLocalStorageKey('settings'));
        var editingFile = settingsObj['editingFile'];
        var configName = settingsObj['editingConfig'];
        var updatedData = editor.getValue();
        esvJSON(updatedData).then(function (output) {
          settingsObj['files'][configName][editingFile] = output;
          LS.sO(NormalizerService.generateLocalStorageKey('settings'), settingsObj);
        });
        var showError = true;
        if (serverData.settings.role === PBConstants.ENABLE_SETTINGS_BTNS_ROLE){
          showError = false;
        }
        Service.saveConfigFile(editingFile, updatedData, configName, showError).done(function (
          data,
          textStatus,
          jqXHR
        ) {
          if (jqXHR.status === PBConstants.SUCCESS_CODE) {
            hideConfigurationNotSaved();
            toastr.success(i18next.t('constantMsgs.configSuccess'));
          } else {
            toastr.info(i18next.t('constantMsgs.ajaxError'));
          }
        });
      }
    });
    editor.focus();
    // For some config files there is no get functionality only save functionality is there.
    handleEditorContentChange(editor);
    var lister = document.querySelectorAll('#acetabs > ul > li > a');
    var lister2 = document.querySelectorAll(
      '#configTabContainer > ul > li > a'
    );
    var lister3 = document.querySelectorAll(
      '#configTabContainer > .tab-content > .tab-pane'
    );
    lister.forEach(function (x) {
      if (

        x.innerHTML
          .toLowerCase()
          .match(editingFile.toLowerCase())
      ) {
        x.classList.add('active');
      } else {
        x.classList.remove('active');
      }
    });
    toggleConfigVisibility();
    if (settingsObj['editingConfig'] === undefined){
        return;
    }
    storeConfigLoadForEdit(settingsObj['editingConfig']);
  }

  /**
   * When a user changes some thing in ace editor then show not saved error message.
   * @param {*} editor
   */
  function handleEditorContentChange(editor) {
    editor.getSession().on('change', function () {
      if (!silent) {
        $('.show-save-message')
        .show()
        .addClass('not-saved');
      }
    });
  }

  /**
   * Set ace editor height
   */
  function setConfigurationContainerHeight() {
    if ($(window).innerWidth() >= PBConstants.MIN_RESOLUTION_FOR_FIXED_HEADERS) {
      var footerHeight = $('.footer').outerHeight() + PBConstants.HEIGHT_REMOVE_TO_FIX_SCROLLBAR;
      // For displaying scrollbar for ace editor body in higher resolution and adding margin top for config header and config file tabs.
      var occupiedHeight =
        $('.editor-container').offset().top + footerHeight;
      var remainingHeight = $(window).outerHeight() - occupiedHeight;
      $('.editor-container').css('height', remainingHeight + 'px');
      editor ? editor.resize() : '';
    }
  }


  /** ******************************
   * END of page load callbacks
   *******************************/

  /**
   * Get tables summary.
   */
  function getTablesInfo() {

    let summaryTables = ['pairs-log', 'pending-log', 'dca-log', 'sales-log'];
    let returnObj = {};

    // Loop over the tables and get summary values.
    for (let i = 0; i < summaryTables.length; i++) {
      let summaryTableData, currentValueTmp;
      let currentValue = 0, boughtCost = 0;

      switch (summaryTables[i]) {
        case 'pairs-log':
          summaryTableData = pairsLog.concat(watchModeLog);
          break;
        case 'pending-log':
          summaryTableData = pendingLog;
          break;
        case 'dca-log':
          summaryTableData = dcaLog;
          break;
        case 'sales-log':
          summaryTableData = salesLog.data;
          break;
      }

      summaryTableData = summaryTableData ? summaryTableData : [];
      if (summaryTables[i] === 'sales-log') {
        for (let j = 0; j < summaryTableData.length; j++) {
          let currentObjSalesLog = summaryTableData[j];
          currentValueTmp = currentObjSalesLog.soldValue;
          currentValue += DataTableHelper.getCurrentValueBsdOnfee(currentObjSalesLog, currentValueTmp);
          boughtCost += currentObjSalesLog.totalCost;
        }
      } else {
        for (let k = 0; k < summaryTableData.length; k++) {
          let currentObj = summaryTableData[k];
          currentValueTmp = currentObj.currentValue;
          if (summaryTables[i] === 'pairs-log' && currentObj.avgPrice === 0) {
            // If bought cost for pairs log is zero, then we are not considering current value for the respective coin.
            currentValueTmp = 0;
          }
          currentValue += DataTableHelper.getCurrentValueBsdOnfee(currentObj, currentValueTmp);
          boughtCost += currentObj.totalCost;
        }
      }
      returnObj[summaryTables[i]] = {
        currentValueConvertedValue: DataTableHelper.normalizeValueWithCommas(currentValue,'display','','','', currentMarketPrecision),
        boughtCostConvertedValue: DataTableHelper.normalizeValueWithCommas(boughtCost, 'display','','','', currentMarketPrecision),
        currentValue: currentValue,
        boughtCost: boughtCost
      };
    }
    return returnObj;
  }

  function addUTCTimer() {
    var currentTime = DateTimeHelper.getCurrentTimeZoneTime(
      serverData.timeZoneOffset
    );
    $currentTime.html(currentTime);
    setTimeout(addUTCTimer, 500);
  }

  function updateUptime() {
    if(!propertiesLog.hasOwnProperty('upTime')) {
      return
    }
    let uptime = DateTimeHelper.getUptimeForBot(propertiesLog.upTime);
    $uptime.html(uptime);
  }

  function getUrlVars() {
    var vars = [],
      hash;
    var hashes = window.location.href
      .slice(window.location.href.indexOf('?') + 1)
      .split('&');
    for (var i = 0; i < hashes.length; i++) {
      hash = hashes[i].split('=');
      vars.push(hash[0]);
      vars[hash[0]] = hash[1];
    }
    return vars;
  }

  function reloadPageOnInactivity(duration) {
    var events = [
      'onclick',
      'onmousemove',
      'onmousedown',
      'ontouchstart',
      'onload',
      'onscroll',
      'onkeypress'
    ];
    var lastActiveTime = new Date();
    for (var i = 0; i !== events.length; i++) {
      document[events[i]] = trackInactivity;
    }

    function trackInactivity() {
      lastActiveTime = new Date();
    }

    function refreshPage() {
      if (duration * 60 < getTimeDiffInSeconds(lastActiveTime)) {
        location.reload();
      }
      setTimeout(refreshPage, 1000);
    }

    function getTimeDiffInSeconds(lastActiveTime) {
      return (new Date().getTime() - lastActiveTime.getTime()) / 1000;
    }

    refreshPage();
  }

  function updateStatsBasedOnServerData() {
    if (propertiesLog.hasOwnProperty('licenseType') && (propertiesLog.licenseType.toLowerCase() === 'pro' || propertiesLog.licenseType.toLowerCase() === 'advanced' || propertiesLog.licenseType.toLowerCase() === 'paper')) {
      $statsMenu.show();
      $('#statsMobile').show();
    } else {
      $statsMenu.hide();
      $('#statsMobile').hide();
    }

    if(propertiesLog.hasOwnProperty('isPTFeederEnabled') && propertiesLog.isPTFeederEnabled) {
      $('#ptfDesktop').show();
      $('#ptfMobile').show();
    } else {
      $('#ptfDesktop').hide();
      $('#ptfMobile').hide();
    }
  }
});
