/** ******************************************************
 * Contains all the code related to API calls, normalization
 * of data received from the API, and configuration of
 * plugins.
 *
 * © ProfitTrailer
 ********************************************************/
var settingsDetailsObj = {
  enableAllBtns : true
};
var disableBtnTooltip = '';
var SOMO_ON = '1';
var SOMO_OFF = '2';
var SOMO_DISABLED = '0';

var Services = function(
  baseAPI,
  marketCapAPI,
  currenciesAPI,
  globalMarketCapAPI
) {
  var BASE_API = baseAPI;
  var MARKET_CAP_API_URL = marketCapAPI;
  var CURRENCIES_API = currenciesAPI;
  var GLOBAL_MARKET_CAP_URL = globalMarketCapAPI;

  var DEFAULT_REQUEST_OBJ = {
    cache: false,
    crossDomain: true,
    dataType: 'json'
  };

  var CONFIG_REQUEST_OBJ = jQuery.extend(
    {
      beforeSend: function() {
        $('.tab-specific-loading').show();
      },
      complete: function() {
        $('.tab-specific-loading').hide();
      }
    },
    DEFAULT_REQUEST_OBJ
  );

  var NormalizerService = Services.Normalizer();

  function getAccountDetails() {
    return $.ajax(
      jQuery.extend(
        {
          url: BASE_API + '/api/v2/account/settings',
          global: false
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function updateLastSeenForNews() {
    return $.ajax(
      jQuery.extend(
        {
          url: BASE_API + '/api/v2/newsfeed/mark',
          global: false,
          type: 'POST',
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function getCurrencies() {
    return $.ajax(
      jQuery.extend(
        {
          url: CURRENCIES_API,
          global: false
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function getAPILogs() {
    return $.ajax(
      jQuery.extend(
        {
          url: BASE_API + '/api/v2/errors/log',
          global: false
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function getPTFeederLogs() {
      return $.ajax(
          jQuery.extend(
              {
                  url: BASE_API + '/api/v2/feeder/log',
                  global: false
              },
              DEFAULT_REQUEST_OBJ
          )
      );
  }

  function getFeed(apiFeed) {
    return $.ajax(
      jQuery.extend(
        {
          url: apiFeed,
          global: false
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function getMarketCap(currency, limit) {
    return $.ajax(
      jQuery.extend(
        {
          url: MARKET_CAP_API_URL + currency + '&price_change_percentage=1h,24h,7d&&per_page=' + limit,
          global: false
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function getGlobalMarketCap(currency) {
    return $.ajax(
      jQuery.extend(
        {
          url: GLOBAL_MARKET_CAP_URL,
          global: false
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function getMarketTrend() {
    return $.ajax(
      jQuery.extend(
        {
          url: BASE_API + '/private?command=returnTicker',
          global: false
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function stopProfitTrailer() {
    return $.ajax(
      jQuery.extend(
        {
          url: BASE_API + '/stop',
          global: false
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }
  function resetProfitTrailer() {
    return $.ajax(
      jQuery.extend(
        {
          url: BASE_API + '/settings/test/reset',
          global: false
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function publishActiveSetting() {
    return $.ajax(
      jQuery.extend(
        {
          url: BASE_API + '/settings/config/publish',
          type: 'POST',
          global: false
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function getConfigFile(filename, configName) {
    return $.ajax(
      jQuery.extend({}, CONFIG_REQUEST_OBJ,
        {
          url:
            BASE_API +
            '/settings/load?fileName=' +
            filename +
            '&configName=' +
            configName,
          global: false,
          cache: false,
          async: true,
          type: 'GET',
          dataType: 'text',

        }
      )
    );
  }

  function switchConfigFile(configName) {
    return $.ajax(
      jQuery.extend(
        {
          url: BASE_API + '/settings/config/switch?&configName=' + configName,
          global: false,
          type: 'POST',
          complete: function(xhr) {
            if(xhr.status === 200){
            storeConfigLoadForEdit(configName);
            } else{
              Swal.fire(i18next.t('alertMsgs.errorswitching'));
            }
          }
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function savePassword(formData) {
    return $.ajax(
      jQuery.extend(
        {
          url: BASE_API + '/login/changePassword',
          global: false,
          type: 'POST',
          data: formData,
          complete: function(xhr) {
            if(xhr.status === 204){
              toastr.success(i18next.t(xhr.statusText));
            } else{
              toastr.info(i18next.t(xhr.statusText));
            }
          }
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function removeConfigFile(configName) {
    return $.ajax(
      jQuery.extend({}, DEFAULT_REQUEST_OBJ, {
        url: BASE_API + '/settings/config/delete?&configName=' + configName,
        global: false,
        type: 'POST'
      })
    );
  }

  function saveConfigFile(filename, fileData, configName, showError, eCfg) {
    return $.ajax(
      jQuery.extend({}, CONFIG_REQUEST_OBJ, {
        url:
          BASE_API +
          '/settings/save?fileName=' +
          filename +
          '&configName=' +
          configName,
        data: fileData,
        dataType: 'text',
        contentType: 'text/plain',
        global: false,
        type: 'POST',
        success: function() {
          toastr.success(i18next.t('constantMsgs.configSuccess'));
        },
        error: function() {
          if (showError){
            toastr.success(i18next.t('constantMsgs.ajaxError'));
          }
        },
        complete: function(xhr) {
          if(eCfg){
            if(xhr.status === 200){
            storeConfigLoadForEdit(configName);
            }
          }
        },
      })
    );
  }

  function saveAll(configName, pairsData, dcaData, indicatorsData) {
    return $.ajax(
      jQuery.extend({}, CONFIG_REQUEST_OBJ, {
        url:
          BASE_API +
          '/settings/saveAll?configName='+configName,
          data: '&pairsData='+encodeURIComponent(pairsData)+'&dcaData='+encodeURIComponent(dcaData)+'&indicatorsData='+encodeURIComponent(indicatorsData),
          // data: "{'configName' : " + configName + ", 'pairsData': " + encodeURIComponent(pairsData) + ", 'dcaData': " + encodeURIComponent(dcaData) + ", 'indicatorsData': " + encodeURIComponent(indicatorsData) + "}",
          dataType: 'text',
          global: false,
        type: 'POST',
        complete: function(xhr) {
          if(xhr.status === 200){
          toastr.success(i18next.t('constantMsgs.configSuccess'));
          storeConfigLoadForEdit(configName);
          }
        }
      })
    );
  }

  function saveSOM(value) {
    return $.ajax(
      jQuery.extend({}, CONFIG_REQUEST_OBJ, {
        url: BASE_API + '/settings/sellOnlyMode?type=&enabled=' + value,
        global: false,
        dataType: 'text'
      })
    );
  }

  function saveSOMO(value) {
    var enabledURL = value !== '' ? '?enabled=' + value : '';
    return $.ajax(
      jQuery.extend({}, CONFIG_REQUEST_OBJ, {
        url: BASE_API + '/api/v2/globalSellOnlyMode' + enabledURL,
        data: value,
        global: false,
        dataType: 'text',
        type: 'POST'
      })
    );
  }

    function savePTNSettings(value) {
        return $.ajax(
            jQuery.extend({}, DEFAULT_REQUEST_OBJ, {
                url:
                BASE_API +
                '/settings/savePTNSettings',
                data: value,
                dataType: 'text',
                contentType: 'application/json',
                global: false,
                type: 'POST'
            })
        );
    }

  function buyOrSellPair(buyOrSell, pairName, executionType, showError) {
    return $.ajax(
      jQuery.extend(
        {
          url: BASE_API + '/action/' + buyOrSell + '?currencyPair=' + pairName + '&executionType=' + executionType,
          global: false,
          type: 'POST',
          success: function() {
            Swal.fire(i18next.t('alertMsgs.submittedOrder', {buyOrSell: buyOrSell, pairname:pairName}));
          },
          error: function(xhr, ajaxOptions, thrownError) {
            if (xhr.status === 404 || showError) {
              Swal.fire(i18next.t('alertMsgs.endPointNotImplemented') + thrownError);
            }
          }
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function enterPending(pairName, targetPrice, showError) {
    return $.ajax(
      jQuery.extend(
        {
          url: BASE_API + '/action/pending?currencyPair=' + pairName + '&targetPrice=' + targetPrice,
          global: false,
          type: 'POST',
          success: function() {
            Swal.fire(i18next.t('alertMsgs.submittedOrder', {pairname: pairName}));
          },
          error: function(xhr, ajaxOptions, thrownError) {
            if (xhr.status === 404 || showError) {
              Swal.fire(i18next.t('alertMsgs.endPointNotImplemented') + thrownError);
            }
          }
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

    /**
     * Requests for api/v2/ routes
     */
    function getPossibleBuyLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/pbl',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getPairsLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/pairs',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getWatchModeLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/watchmode',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getDcaLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/dca',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getPendingLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/pending',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getBuyLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/buys',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getSalesLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/sales',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getDustLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/dust',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getReservedCoins() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/reserved',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getStatsLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/stats',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getPropertiesLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/properties',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getNotificationsLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/notification',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getMiscLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/misc',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getHeartbeatsLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/heartbeats',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function getAccountLog() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/account/settings',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }

    function getReversalCoins() {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/data/reversal',
                global: false
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function cancelReversal(reversalId, showError) {
        return $.ajax(
            jQuery.extend({
                url: BASE_API + '/api/v2/reversal/remove?reversalPositionId=' + reversalId,
                global: false,
                type: 'POST',
                complete: function(xhr) {
                    if(xhr.status === 304) {
                        Swal.fire({
                            title: i18next.t('alertMsgs.somethingWrong'),
                            text: xhr.statusText
                        });
                    } else {
                        Swal.fire(i18next.t('reversal.success'));
                    }
                },
                error: function(xhr, ajaxOptions, thrownError) {
                    if (xhr.status === 404 || showError) {
                        Swal.fire(i18next.t('alertMsgs.endPointNotImplemented') + thrownError);
                    }
                }
            }, DEFAULT_REQUEST_OBJ)
        );
    }
    function releaseAmount(pair, amount, showError) {
        return $.ajax(
            jQuery.extend(
                {
                    url: BASE_API + '/api/v2/balance/release?amount='+ amount +'&currencyPair=' + pair,
                    global: false,
                    type: 'POST',
                    complete: function(xhr) {
                        if(xhr.status === 304) {
                            Swal.fire({
                                title: i18next.t('alertMsgs.somethingWrong'),
                                text: xhr.statusText
                            });
                        } else {
                            Swal.fire(i18next.t('releaseAmount.successText'));
                        }
                    },
                    error: function(xhr, ajaxOptions, thrownError) {
                        if (xhr.status === 404 || showError) {
                            Swal.fire(i18next.t('alertMsgs.endPointNotImplemented') + thrownError);
                        }
                    }
                },
                DEFAULT_REQUEST_OBJ
            )
        );
    }
  function reserveAmount(pair, amount, showError) {
      return $.ajax(
          jQuery.extend(
              {
                  url: BASE_API + '/api/v2/balance/reserve?amount='+ amount +'&currencyPair=' + pair,
                  global: false,
                  type: 'POST',
                  complete: function(xhr) {
                      if(xhr.status === 304) {
                          Swal.fire({
                              title: i18next.t('alertMsgs.somethingWrong'),
                              text: xhr.statusText
                          });
                      } else {
                          Swal.fire(i18next.t('reserveAmount.successText'));
                      }
                  },
                  error: function(xhr, ajaxOptions, thrownError) {
                      if (xhr.status === 404 || showError) {
                          Swal.fire(i18next.t('alertMsgs.endPointNotImplemented') + thrownError);
                      }
                  }
              },
              DEFAULT_REQUEST_OBJ
          )
      );
  }

  function addBuyOrSalesRecord(formData, endpoint, showError) {
      let url = '';
      let modalToClose = '';
      if (endpoint === 'buy') {
          url = '/action/logBuy?currencyPair=' + formData.buyLogCurrencyPair + '&boughtAmount=' + formData.buyLogAmount + '&boughtPrice=' + formData.buyLogAvgPrice;
          modalToClose = 'addBuyRecord';
      } else {
          url = '/action/logSell?currencyPair=' + formData.salesLogCurrencyPair + '&soldAmount=' + formData.salesLogAmount + '&soldPrice=' + formData.salesLogSoldPrice + '&boughtPrice=' + formData.salesLogAvgPrice;
          modalToClose = 'addSalesRecord';
      }

      return $.ajax(
          jQuery.extend(
              {
                  url: BASE_API + url,
                  global: false,
                  type: 'POST',
                  complete: function(xhr) {
                      if(xhr.status === 304) {
                          Swal.fire({
                              title: i18next.t('alertMsgs.somethingWrong'),
                              text: xhr.statusText
                          });
                      } else {
                          $('#' + modalToClose).modal('hide');
                          Swal.fire(i18next.t('alertMsgs.recordAdded'));
                      }
                  },
                  error: function(xhr, ajaxOptions, thrownError) {
                      if (xhr.status === 404 || showError) {
                          Swal.fire(i18next.t('alertMsgs.endPointNotImplemented') + thrownError);
                      }
                  }
              },
              DEFAULT_REQUEST_OBJ
          )
      );
  }

  function cancelPending(pairName, orderNumber, showError) {
    return $.ajax(
      jQuery.extend(
        {
          url: BASE_API + '/action/cancel?currencyPair='+pairName+'&orderNumber='+orderNumber,
          global: false,
          type: 'POST',
          complete: function() {

            Swal.fire(i18next.t('alertMsgs.cancelledOrderNumber', {orderNumber:orderNumber}));

          },
          error: function(xhr, ajaxOptions, thrownError) {
            if (xhr.status === 404 || showError) {
                Swal.fire(i18next.t('alertMsgs.endPointNotImplemented') + thrownError);
            }
          }
        },
        DEFAULT_REQUEST_OBJ
      )
    );
  }

  function getCurrentMarketPrecision (precision) {
    precision = precision || PBConstants.DEFAULT_PRECISION;
    // If market is USDT then we should add 2 decimals precision.

    return precision;
  }

  return {
    getMarketCap: getMarketCap,
    getMarketTrend: getMarketTrend,
    getAPILogs: getAPILogs,
    getPTFeederLogs: getPTFeederLogs,
    buyOrSellPair: buyOrSellPair,
    getConfigFile: getConfigFile,
    saveConfigFile: saveConfigFile,
    saveAll:saveAll,
    cancelPending:cancelPending,
    enterPending: enterPending,
    addBuyOrSalesRecord: addBuyOrSalesRecord,
    switchConfigFile: switchConfigFile,
    savePassword: savePassword,
    removeConfigFile: removeConfigFile,
    getCurrentMarketPrecision: getCurrentMarketPrecision,
    stopProfitTrailer: stopProfitTrailer,
    resetProfitTrailer:resetProfitTrailer,
    publishActiveSetting: publishActiveSetting,
    getCurrencies: getCurrencies,
    Normalizer: Services.Normalizer(),
    Configurations: Services.Configurations(),
    saveSOM: saveSOM,
    saveSOMO: saveSOMO,
    getGlobalMarketCap: getGlobalMarketCap,
    getFeed: getFeed,
    savePTNSettings: savePTNSettings,
    getAccountDetails: getAccountDetails,
    updateLastSeenForNews: updateLastSeenForNews,
      // New End Points - Check above this line
      reserveAmount: reserveAmount,
      releaseAmount: releaseAmount,
      getPossibleBuyLog: getPossibleBuyLog,
      getPairsLog: getPairsLog,
      getWatchModeLog: getWatchModeLog,
      getDcaLog: getDcaLog,
      getPendingLog: getPendingLog,
      getBuyLog: getBuyLog,
      getSalesLog: getSalesLog,
      getDustLog: getDustLog,
      getReservedCoins: getReservedCoins,
      getReversalCoins: getReversalCoins,
      getStatsLog: getStatsLog,
      getPropertiesLog: getPropertiesLog,
      getNotificationsLog: getNotificationsLog,
      getMiscLog: getMiscLog,
      getHeartbeatsLog: getHeartbeatsLog,
      getAccountLog: getAccountLog,
      cancelReversal:cancelReversal,
  };
};

// Configurations.
Services.Configurations = function() {
  function setToastrOptions() {
    toastr.options = {
      closeButton: true,
      newestOnTop: true,
      positionClass: 'toast-top-right',
      showDuration: '300',
      hideDuration: '1000',
      timeOut: '5000',
      extendedTimeOut: '1000',
      showEasing: 'swing',
      hideEasing: 'linear',
      showMethod: 'fadeIn',
      hideMethod: 'fadeOut',
      preventDuplicates: true
    };
  }

  return {
    setToastrOptions: setToastrOptions
  };
};

// Data normalization
Services.Normalizer = function() {
  var MAX_HEARTBEATS = 12;
  var ICONS = {
    'cache heartbeat': {
      class: 'mdi-heart-pulse',
      'dark-theme-color': '#3bafda',
      'light-theme-color': '#3bafda'
    },
    'dca heartbeat': {
      class: 'mdi-heart-pulse',
      'dark-theme-color': 'rgba(133, 187, 101, 0.75)',
      'light-theme-color': 'rgba(133, 187, 101, 0.75)'
    },
    'pairs heartbeat': {
      class: 'mdi-heart-pulse',
      'dark-theme-color': '#ef5350',
      'light-theme-color': '#ef5350'
    },
    'detected configuration changes': {
      class: 'mdi-settings',
      'dark-theme-color': 'rgba(176, 196, 222, 0.75)',
      'light-theme-color': '#4B8DE0'
    },
    default: {
      class: 'mdi-alert-circle-outline',
      'dark-theme-color': '#FF9900',
      'light-theme-color': '#4B8DE0'
    },
    'api issues': {
      class: 'mdi-desktop-tower',
      'dark-theme-color': 'rgba(176, 196, 222, 0.75)',
      'light-theme-color': '#4B8DE0'
    },
    'something wrong with bot': {
      class: 'mdi-robot',
      'dark-theme-color': 'rgba(176, 196, 222, 0.75)',
      'light-theme-color': '#4B8DE0'
    },
    'invalid strategy selected': {
      class: 'mdi-dice-1',
      'dark-theme-color': 'rgba(176, 196, 222, 0.75)',
      'light-theme-color': '#4B8DE0'
    }
  };

  var hashString = '';

  // To remove the unused data which we are receiving from API.
  function removeUnusedData(response) {
    delete response.dcaBackupLogData;
    delete response.dcaBuyOrdersHistory;
    delete response.orderHistory;
    delete response.storedAverageMap;
    delete response.testModeBalanceMap;
    delete response.testModeTradeHistoryMap;
    return response;
  }

  function getIconType(text) {
    var lowerText = text.toLowerCase();
    var icon = null;
    if (ICONS.hasOwnProperty(lowerText)) {
      icon = ICONS[lowerText];
    } else {
      icon = ICONS.default;
    }
    return icon;
  }

  // TODO: This function should not be here, need to be moved to correct place.
  function settingsDetails(data) {
    if (data && data.settings) {
      var dataSettings = data.settings;
      var isPasswordSet = dataSettings.passwordSet;

      // If password is off then disable all the buttons.
      settingsDetailsObj.enableAllBtns = isPasswordSet;
      // Enable top config buttons only if password is on and role is not "M".
      settingsDetailsObj.enableConfigBtns = isPasswordSet && dataSettings.role !== PBConstants.ENABLE_SETTINGS_BTNS_ROLE;
      // Enable "Stop ProfitTrailer" button only if password is on and enableStopBtn is true.
      settingsDetailsObj.enableStopPTBtn = isPasswordSet && dataSettings.enableShutdown;
      // Enable "Change Passwords" button only if password is on and role is "A".
      settingsDetailsObj.enableChangePasswordBtn = isPasswordSet && dataSettings.role === PBConstants.ENABLE_CHNGE_PSWRD_ROLE;
      settingsDetailsObj.enableChangeConfigBtn = isPasswordSet;

      // If password is off then show password not set tooltip.
      if (!isPasswordSet) {
        disableBtnTooltip = 'settingInfo.noPswrd.title';
      } else {
        disableBtnTooltip = '';
      }
      settingsDetailsObj.disableBtnTooltip = disableBtnTooltip;
      settingsDetailsObj.disableTopBtnsTooltip = disableBtnTooltip || 'settingInfo.incorrectRole.title';
    }
    return settingsDetailsObj;
  }

  function marketCap(data, marketCapCurrency) {
    if (data === null) {
      return false;
    }
    try {
      var percChange1h = 0;
      var percChange24h = 0;
      var percChange7d = 0;
      var totalRecords = data ? data.length : 0;

      for (var i = 0; i !== totalRecords; ++i) {
        if (data[i].symbol === marketCapCurrency) {
          // exclude bitcoin in the trend
          continue;
        }
        percChange1h += +data[i].price_change_percentage_1h_in_currency;
        percChange24h += +data[i].price_change_percentage_24h_in_currency;
        percChange7d += +data[i].price_change_percentage_7d_in_currency;
      }

      if (totalRecords) {
        percChange1h = percChange1h / totalRecords;
        percChange24h = percChange24h / totalRecords;
        percChange7d = percChange7d / totalRecords;
      }
    } catch (e) {
      console.log('Error while processing data from market cap API.');
      console.log(e);
      return false;
    }

    return {
      percChange1h: percChange1h,
      percChange24h: percChange24h,
      percChange7d: percChange7d
    };
  }

  function normalizeCurrencies(data, currencies) {
    var currentCurrency = data.settings.currency;
    var isBaseCurrency =
      currentCurrency && currencies.hasOwnProperty(currentCurrency)
        ? false
        : true;
    data.currency = isBaseCurrency
      ? PBConstants.BASE_CURRENCY
      : currentCurrency;
    data.currencyValue = isBaseCurrency ? 1 : currencies[currentCurrency];
    if (
      !isBaseCurrency &&
      PBConstants.CURRENCY_SYMBOLS.hasOwnProperty(currentCurrency)
    ) {
      data.currencySymbol = PBConstants.CURRENCY_SYMBOLS[currentCurrency].symbol;
      data.currencyName = i18next.t(PBConstants.CURRENCY_SYMBOLS[currentCurrency].name);
    } else {
      data.currencySymbol = PBConstants.BASE_CURRENCY_SYMBOL;
      data.currencyName =  i18next.t(PBConstants.BASE_CURRENCY_NAME);
    }
    return data;
  }

  function segregateNotifications(data) {
    var notifications = data && data.notifications ? data.notifications : [];
    var heartbeats = data && data.heartbeats ? data.heartbeats : [];
    var heartbeatsLength = heartbeats.length;
    var notificationsLength = notifications.length;
    var currentNotificationIndex = notifications.length - 1;
    var nrmlsdNotifications = [];
    var heartBeats = [];
    var currentHeartbeatIndex = heartbeatsLength - 1;
    var currentHeartbeat;
    var i;
    // To loop over heartbeat array only if heartbeat array's length is greater than zero.
    if (heartbeatsLength !== 0) {
      i = 0;
      // Calcuate the number of times the looping occurs based on length of heartbeat array.
      var maxHeartBeatLength =
        heartbeatsLength <= MAX_HEARTBEATS ? heartbeatsLength : MAX_HEARTBEATS;
      while (i < maxHeartBeatLength) {
        currentHeartbeat = heartbeats[currentHeartbeatIndex];
        currentHeartbeat.icon = getIconType(currentHeartbeat.heartbeat);
        heartBeats.push(currentHeartbeat);
        currentHeartbeatIndex--;
        i++;
      }
    }
    // To loop over notification array only if notification array's length is greater than zero.
    if (notificationsLength !== 0) {
      // We are looping only through X notifications to avoid performance issues that might arise if we loop through all the notifications
      i = 0;
      var maxLimit =
        notificationsLength < PBConstants.NOTIFICATIONS.NOTIFICATIONS_CUT_OFF
          ? notificationsLength
          : PBConstants.NOTIFICATIONS.NOTIFICATIONS_CUT_OFF;
      while (i < maxLimit) {
        var currentNotification = notifications[currentNotificationIndex];
        currentNotification.icon = getIconType(
          currentNotification.notification
        );
        // Pushing current notification in notification array.
        nrmlsdNotifications.push(currentNotification);
        currentNotificationIndex--;
        i++;

        // If we found 20 notifications
        // then we can stop looping as we are showing showing max 20 notifications.
        if (nrmlsdNotifications.length > PBConstants.NOTIFICATIONS.MAX) {
          break;
        }
      }
    }

    data.nrmlsdNotifications = nrmlsdNotifications;
    data.heartBeats = heartBeats;
    return data;
  }

  /**
   * If market is USDT then we should consider market as BTC.
   * @param {*} market
   */
  function normalizeMarket(market) {
    market =
      market === PBConstants.HIDE_PROFIT_MARKET_AND_ESTIMATED_USD
        ? 'BTC'
        : market;
    return market;
  }

  /**
   * It checks whether data.stats is object or not.
   * @param {*} data
   */
  function normalizeStats(data) {
    if ($.isEmptyObject(data.stats)) {
      return {};
    } else {
      return data.stats;
    }
  }

  function normalizeValuesWithCommas(value, precision, noPrecision) {
    precision = precision || PBConstants.DEFAULT_PRECISION;
    if (noPrecision) {
      return DataTableHelper.normalizeValueWithCommas(
        value,
        'display',
        '',
        '',
        true
      );
    } else {
      return DataTableHelper.normalizeValueWithCommas(
        value,
        'display',
        '',
        '',
        '',
        precision
      );
    }
  }

  function normalizeSummaryData(summary, precision) {
    // Don't modify the original object.
    var summaryTemp = $.extend({}, summary);
    var normalizedSummary = summaryTemp;
    var precisionToUse = precision || PBConstants.DEFAULT_PRECISION;
    precisionToUse = Services().getCurrentMarketPrecision(
      summary.apiMarket,
      precisionToUse
    );
    normalizedSummary.balance = normalizeValuesWithCommas(
      +summaryTemp.balance,
      precisionToUse
    );
    normalizedSummary.DCABalance = normalizeValuesWithCommas(
      +summaryTemp.DCABalance,
      precisionToUse
    );
    normalizedSummary.pairsBalance = normalizeValuesWithCommas(
      +summaryTemp.pairsBalance,
      precisionToUse
    );
    normalizedSummary.reversalReservedBalance = normalizeValuesWithCommas(
      +summaryTemp.reversalReservedBalance, precisionToUse
    );
    normalizedSummary.totalCurrentVal = normalizeValuesWithCommas(
      +summaryTemp.totalCurrentVal,
      precisionToUse
    );
    normalizedSummary.TCVDustValue = normalizeValuesWithCommas(
      +summaryTemp.TCVDustValue,
      precisionToUse
    );
    normalizedSummary.startBalance = normalizeValuesWithCommas(
      +summaryTemp.startBalance,
      precisionToUse
    );
    normalizedSummary.TPVDustValue = normalizeValuesWithCommas(
      +summaryTemp.TPVDustValue,
      precisionToUse
    );
    normalizedSummary.todayProfit = normalizeValuesWithCommas(
      +summaryTemp.todayProfit,
      precisionToUse
    );
    normalizedSummary.yesterdayProfit = normalizeValuesWithCommas(
      +summaryTemp.yesterdayProfit,
      precisionToUse
    );
    normalizedSummary.lastWeekProfit = normalizeValuesWithCommas(
      +summaryTemp.lastWeekProfit,
      precisionToUse
    );
    normalizedSummary.lastWeekMarketVolume = normalizeValuesWithCommas(
      +summaryTemp.lastWeekMarketVolume
    );
    normalizedSummary.yesterdayMarketVolume = normalizeValuesWithCommas(
      +summaryTemp.yesterdayMarketVolume
    );
    normalizedSummary.todayMarketVolume = normalizeValuesWithCommas(
      +summaryTemp.todayMarketVolume
    );
    normalizedSummary.lastWeekProfitSales = normalizeValuesWithCommas(
      +summaryTemp.totalSalesWeek,
      0,
      true
    );
    normalizedSummary.yesterdayProfitSales = normalizeValuesWithCommas(
      +summaryTemp.totalSalesYesterday,
      0,
      true
    );
    normalizedSummary.todayProfitSales = normalizeValuesWithCommas(
      +summaryTemp.totalSalesToday,
      0,
      true
    );

    for (var prop in normalizedSummary) {
      if (
        typeof normalizedSummary[prop] === undefined ||
        (typeof normalizedSummary[prop] === 'string' &&
          normalizedSummary[prop].toLowerCase() === 'nan')
      ) {
        normalizedSummary[prop] = '--';
      }
    }
    return normalizedSummary;
  }

  function marketTrend(data, marketCapCurrency) {
    var marketTrend = 0;
    var marketTrendAvg = 0;
    try {
      var marketRecords = 0;
      for (var key in data) {
        // Consider coins contains marketCapCurrency
        if (key.indexOf(marketCapCurrency) !== -1) {
          var value = JSON.parse(data[key]);
          marketTrend += +value.percentChange * 100;
          ++marketRecords;
        }
      }
      marketTrendAvg = marketRecords ? marketTrend / marketRecords : 0;
    } catch (e) {
      console.error('Error while processing data from market Trend API');
      console.error(e);
      return false;
    }
    return marketTrendAvg;
  }

    function getTablesRecordsCount(routes, tables, possibleBuyLog, pairsLog, dcaLog, pendingLog, buyLog, salesLog, dustLog, reservedCoins, reversalCoins, watchModeLog) {
        var recordsCount = {};
        for (var i = 0; i < tables.length; i++) {
            var currentTable = tables[i];

            switch (currentTable) {
                case 'possible-buys-log':
                    recordsCount[currentTable] = possibleBuyLog;
                    break;
                case 'pairs-log':
                    recordsCount[currentTable] = (pairsLog + watchModeLog);
                    break;
                case 'dca-log':
                    recordsCount[currentTable] = dcaLog;
                    break;
                case 'pending-log':
                    recordsCount[currentTable] = pendingLog;
                    break;
                case 'buy-log':
                    recordsCount[currentTable] = buyLog;
                    break;
                case 'sales-log':
                    recordsCount[currentTable] = salesLog;
                    break;
                case 'misc-log':
                    recordsCount[currentTable] = dustLog + ' - ' + reservedCoins + ' - ' + reversalCoins;
                    break;
            }
        }
        return recordsCount;
    }

  function getInCurrencyValue(value, data, precision, checkCondition) {
    if (
      (checkCondition && !showInCurrentCurrency(data)) ||
      !data ||
      $.isEmptyObject(data)
    ) {
      return '';
    }
    var marketTmp = data.market;
    var marketPrice = data[marketTmp + 'USDTPrice']
      ? data[marketTmp + 'USDTPrice']
      : 1;
    value = value * marketPrice * data.currencyValue;
    if (precision) {
      value = DataTableHelper.normalizeValueWithCommas(
        value,
        'display',
        '',
        '',
        '',
        precision
      );
    }
    return value;
  }

  function showInCurrentCurrency(data) {
    // If market is USDT and currency is USD then we should not show the current currency value.
    if (
      data &&
      data.market === PBConstants.HIDE_PROFIT_MARKET_AND_ESTIMATED_USD &&
      data.currency === PBConstants.BASE_CURRENCY
    ) {
      return false;
    }
    return true;
  }

  function getMonitoringSummary(data, lowerPrecision) {
    var precision = PBConstants.DEFAULT_PRECISION;
    if (lowerPrecision) {
      precision = 3;
    }
    if (!data.settings) {
      return false;
    }
    var percChange1hClass =
      data.coinMarketCap.percChange1h < 0
        ? PBConstants.NEGATIVE_CLASS_TEXT
        : PBConstants.POSITIVE_CLASS_TEXT;
    var percChange24hClass =
      data.coinMarketCap.percChange24h < 0
        ? PBConstants.NEGATIVE_CLASS_TEXT
        : PBConstants.POSITIVE_CLASS_TEXT;
    var percChange7dClass =
      data.coinMarketCap.percChange7d < 0
        ? PBConstants.NEGATIVE_CLASS_TEXT
        : PBConstants.POSITIVE_CLASS_TEXT;

    var percChange24hBtcClass =
      data.BTCUSDTPercChange < 0
        ? PBConstants.NEGATIVE_CLASS_TEXT
        : PBConstants.POSITIVE_CLASS_TEXT;
    var marketTrendAvgClass =
      data.marketTrendAvg < 0
        ? PBConstants.NEGATIVE_CLASS_TEXT
        : PBConstants.POSITIVE_CLASS_TEXT;

    var normalizedMarket = data.normalizedMarket;
    var normalizedMarketPrice = data[normalizedMarket + 'USDTPrice']
      ? data[normalizedMarket + 'USDTPrice']
      : 0;
    var normalizedMarketPercChange = data[normalizedMarket + 'USDTPercChange']
      ? data[normalizedMarket + 'USDTPercChange']
      : 0;
    var normalizedMarketPercChangeClass =
      normalizedMarketPercChange < 0
        ? PBConstants.NEGATIVE_CLASS_TEXT
        : PBConstants.POSITIVE_CLASS_TEXT;

    var marketPriceClass =
      data.market === PBConstants.HIDE_MARKET_PRICE_ROW_COMPARISION
        ? 'hide'
        : 'show';

    var returnDataTmp = {
      balance: data.realBalance,
      pairsBalance: data.pairsBalance,
      DCABalance: data.DCABalance,
      reversalReservedBalance: data.reversalReservedBalance,
      todayProfit: data.stats.totalProfitToday,
      totalSalesWeek: data.stats.totalSalesWeek,
      totalSalesYesterday: data.stats.totalSalesYesterday,
      totalSalesToday: data.stats.totalSalesToday,
      lastWeekMarketVolume: data.stats.totalVolumeWeek,
      yesterdayMarketVolume: data.stats.totalVolumeYesterday,
      todayMarketVolume: data.stats.totalVolumeToday,
      yesterdayProfit: data.stats.totalProfitYesterday,
      lastWeekProfit: data.stats.totalProfitWeek,
      totalCurrentVal: parseFloat(
        (
          data.totalPairsCurrentValue +
          data.totalDCACurrentValue +
          data.totalPendingCurrentValue +
          data.realBalance
        ).toFixed(precision)
      ).toFixed(precision),
      startBalance: parseFloat(data.startBalance ? data.startBalance.toFixed(precision) : 0).toFixed(
        precision
      ),
      percChange1h:
        parseFloat(data.coinMarketCap.percChange1h.toFixed(2)).toFixed(2) +
        ' %',
      percChange24h:
        parseFloat(data.coinMarketCap.percChange24h.toFixed(2)).toFixed(2) +
        ' %',
      percChange7d:
        parseFloat(data.coinMarketCap.percChange7d.toFixed(2)).toFixed(2) +
        ' %',
      percBtcChange24h:
        checkAndAddPrecision(data.BTCUSDTPercChange * 100, 2) + ' %',
      normalizedMarketPercChange:
        checkAndAddPrecision(normalizedMarketPercChange * 100, 2) + ' %',
      marketTrendAvg: checkAndAddPrecision(data.marketTrendAvg, 2) + ' %',
      percChange24hBtcClass: percChange24hBtcClass,
      normalizedMarketPercChangeClass: normalizedMarketPercChangeClass,
      priceUsd: normalizeValuesWithCommas(data.BTCUSDTPrice, 2),
      normalizedMarketPrice: normalizeValuesWithCommas(
        normalizedMarketPrice,
        2
      ),
      marketTrendAvgClass: marketTrendAvgClass,
      apiMarket: data.market,
      normalizedMarket: normalizedMarket,
      percChange1hClass: percChange1hClass,
      percChange24hClass: percChange24hClass,
      percChange7dClass: percChange7dClass,
      marketTitle:
        normalizedMarket + 'USD ' + i18next.t('summaryItems.price.title'),
      pendingOrderTime: data.pendingOrderTime,
      balMarketPrice: getInCurrencyValue(data.realBalance, data, 2),
      marketPriceClass: marketPriceClass,
      currencySymbol: data.currencySymbol,
      currencySymbolTitle: data.currencyName,
      estimatedValueCurrencyText:
        data.currency + i18next.t('monitoringSection.usdFullText'),
      estimatedValueShortText:
        data.currency + i18next.t('monitoringSection.usdShortText'),
      exchange: data.exchange
    };
    var marketPriceValues = {
      TPVMarketPrice: getInCurrencyValue(returnDataTmp.startBalance, data, 2),
      TCVMarketPrice: getInCurrencyValue(
        returnDataTmp.totalCurrentVal,
        data,
        2
      ),
      todayProfitUSDValue: getInCurrencyValue(
        returnDataTmp.todayProfit,
        data,
        2
      ),
      yesterdayProfitUSDValue: getInCurrencyValue(
        returnDataTmp.yesterdayProfit,
        data,
        2
      ),
      lastWeekProfitUSDValue: getInCurrencyValue(
        returnDataTmp.lastWeekProfit,
        data,
        2
      )
    };
    // Normalize profit %
    marketPriceValues.yesterdayProfitPercent = parseFloat(
      data.stats.totalProfitPercYesterday.toFixed(2)
    ).toFixed(2);

    marketPriceValues.todayProfitPercent = parseFloat(
      data.stats.totalProfitPercToday.toFixed(
        2
      )
    ).toFixed(2);

    marketPriceValues.lastWeekProfitPercent = parseFloat(
      data.stats.totalProfitPercWeek.toFixed(
        2
      )
    ).toFixed(2);

    var dustValues = {
      TCVDustValue: normalizeValuesWithCommas(
        +returnDataTmp.totalCurrentVal + data.totalDustCurrentValue
      )
    };

    returnDataTmp = $.extend(returnDataTmp, marketPriceValues, dustValues);
    var returnData = $.extend(returnDataTmp, getSecondHeaderData(data));
    return returnData;
  }

  /**
   * It is to fix .toFixed not defined error
   * this error will occur when value is not an integer.
   * @param {*} value
   * @param {*} precisionTmp
   */
  function checkAndAddPrecision(value, precisionTmp) {
    var returnVal = value ? +value : 0;
    var precision = precisionTmp ? precisionTmp : PBConstants.DEFAULT_PRECISION;
    return DataTableHelper.normalizeValueWithCommas(
      returnVal,
      'display',
      '',
      '',
      '',
      precision
    );
  }

  function addPrecisionForMultipleValues(originalObj, data, precision) {
    for (var key in originalObj) {
      originalObj[key] = checkAndAddPrecision(
        data[originalObj[key]],
        precision
      );
    }
    return originalObj;
  }

  function getClassName(value) {
    var className =
      value < 0
        ? PBConstants.NEGATIVE_CLASS_TEXT
        : PBConstants.POSITIVE_CLASS_TEXT;
    return className;
  }

  function getSecondHeaderData(data) {
    var returnObj = {
      sellOnlyMode: '',
      sellOnlyModeOverride: '',
      sellOnlyModeToolTip: ''
    };
    var isPasswordSet =
      data.settings.hasOwnProperty('passwordSet') && data.settings.passwordSet;
    // If password is off then disable config.
    var isEnableConfig = isPasswordSet &&
      data.settings.hasOwnProperty('enableConfig') &&
      data.settings.enableConfig;
    var somStatus =
      data.settings.hasOwnProperty('sellOnlyMode') &&
      data.settings.sellOnlyMode;
    var somoStatus =
      data.settings.hasOwnProperty('sellOnlyModeOverride') &&
      data.settings.sellOnlyModeOverride;
    var somoDrpDwnValue = SOMO_DISABLED;
    // SOM/SOMO true Show Forced ON
    // SOMO true and SOM false Show Forced OFF
    // SOMO false Show Disabled
    if (somoStatus && somStatus) {
      somoDrpDwnValue = SOMO_ON;
    } else if (!somStatus && somoStatus) {
      somoDrpDwnValue = SOMO_OFF;
    }

    if (data.settings) {
      returnObj = {
        sellOnlyMode: somStatus
          ? i18next.t('constantMsgs.somtrue')
          : i18next.t('constantMsgs.somfalse'),
        sellOnlyModeClass: somStatus
          ? PBConstants.NEGATIVE_TEXT
          : PBConstants.POSITIVE_TEXT,
        sellOnlyModeOverride: somoStatus
          ? i18next.t('constantMsgs.somotrue')
          : i18next.t('constantMsgs.somofalse'),
        somoDropDownValue: data.settings.sellOnlyMode ? 'true' : 'false',
        sellOnlyModeOverrideClass: somoStatus
          ? PBConstants.NEGATIVE_BADGE
          : PBConstants.POSITIVE_BADGE,
        sellOnlyModeToolTip:
          data.settings.sellOnlyMode && data.settings.sellOnlyModeTrigger
            ? data.settings.sellOnlyModeTrigger
            : '',
        configEnabled: isEnableConfig
          ? i18next.t('constantMsgs.onConfig')
          : i18next.t('constantMsgs.offConfig'),
        configEnabledClass: isEnableConfig
          ? PBConstants.POSITIVE_BADGE
          : PBConstants.NEGATIVE_BADGE,
        passwordSet: isPasswordSet
          ? i18next.t('constantMsgs.onPassword')
          : i18next.t('constantMsgs.offPassword'),
        passwordSetClass: isPasswordSet
          ? PBConstants.POSITIVE_BADGE
          : PBConstants.NEGATIVE_BADGE,
        isTestMode: data.settings.hasOwnProperty('testMode')
          ? data.settings.testMode
          : false,
        isTestNet: data.settings.hasOwnProperty('testnet')
          ? data.settings.testnet
          : false,
        isMargin: data.settings.hasOwnProperty('margin')
          ? data.settings.margin
          : false,
        botType: data.settings.hasOwnProperty('licenseType') ? firstLetterUpperCase(data.settings.licenseType) : ''
      };
    }
    return returnObj;
  }

  function firstLetterUpperCase(str) {
    str = str.toLowerCase().replace(/\b[a-z]/g, function (letter) {
      return letter.toUpperCase();
    });
    return str;
  }

  function getDefaultCoinMarket() {
    return {
      percChange1h: 0,
      percChange24h: 0,
      percChange7d: 0
    };
  }

  /* To generate hashed string to prefix with localStorage key to get unique key.
  * Concating origin URL with window pathname's 2nd element to get string for creating hash tag.
  */
  function getHashString() {
    var hash = 0;
    var windowLocation = window.location.origin;
    var windowPath = window.location.pathname;
    var splitPath = windowPath.split('/');
    var concatString = windowLocation;
    // When pathname contains only one element then dont concate the path name.
    if (splitPath.length >1 && splitPath[0]) {
      concatString = concatString + splitPath[1];
    }
    for (var i = 0; i < concatString.length; i++) {
      var character = concatString.charCodeAt(i);
      hash = ((hash<<5)-hash)+character;
      hash = hash & hash;
    }
    hashString = hash;
    return hash;
  }

  function generateLocalStorageKey(keyName) {
    var contentString = '';
    if (hashString === '') {
      contentString = getHashString();
    } else {
      contentString = hashString;
    }
    return contentString + '_' + keyName;
  }

  return {
    marketCap: marketCap,
    marketTrend: marketTrend,
    normalizeSummaryData: normalizeSummaryData,
    getTablesRecordsCount: getTablesRecordsCount,
    getMonitoringSummary: getMonitoringSummary,
    checkAndAddPrecision: checkAndAddPrecision,
    getSecondHeaderData: getSecondHeaderData,
    addPrecisionForMultipleValues: addPrecisionForMultipleValues,
    getDefaultCoinMarket: getDefaultCoinMarket,
    normalizeCurrencies: normalizeCurrencies,
    segregateNotifications: segregateNotifications,
    settingsDetails: settingsDetails,
    removeUnusedData: removeUnusedData,
    getClassName: getClassName,
    normalizeValuesWithCommas: normalizeValuesWithCommas,
    normalizeMarket: normalizeMarket,
    getInCurrencyValue: getInCurrencyValue,
    getHashString: getHashString,
    generateLocalStorageKey: generateLocalStorageKey,
    normalizeStats: normalizeStats
  };
};
