let lastDailyTCVStats = [];
let lastSaleDistribution = [];
let lastDailyStats = [];
let lastMonthlyStats = [];
let lastSellLogData = [];

function resetLastStatsVariables() {
    lastDailyTCVStats = [];
    lastSaleDistribution = [];
    lastDailyStats = [];
    lastMonthlyStats = [];
    lastSellLogData = [];
}



function initializeStats(data, propertiesLog, salesLog) {
    console.log(data)
    if (!data.hasOwnProperty('extra')) {
        return;
    }

    let dailyStats = data.extra.dailyStats.reverse();
    let monthlyStats = data.extra.monthlyStats.reverse();
    let theme = propertiesLog.skin;

    let dailyStatsUnion = _.intersectionWith(dailyStats, lastDailyStats, _.isEqual);
    let monthlyStatsUnion = _.intersectionWith(monthlyStats, lastMonthlyStats, _.isEqual);
    let saleDistributionUnion = _.intersectionWith(data.extra.saleDistribution, lastSaleDistribution, _.isEqual);
    let dailyTCVStatsUnion = _.intersectionWith(data.extra.dailyTCVStats, lastDailyTCVStats, _.isEqual);
    let salesStatsUnion = _.intersectionWith(salesLog.data, lastSellLogData, _.isEqual);

    if (dailyStatsUnion.length !== dailyStats.length) {
        prepareProfitTables(dailyStats, 'dailyProfitTableBody');
        prepareProfitCharts(dailyStats, 'dailyProfitsChart', theme);
        prepareEarnedVsSpentCharts(dailyStats, 'dailySpentEarnedChart', theme);
        prepareBuysVsSalesCharts(dailyStats, 'dailyBuySalesChart', theme);
        lastDailyStats = dailyStats;
    }

    if (monthlyStatsUnion.length !== monthlyStats.length) {
        prepareProfitTables(monthlyStats, 'monthlyProfitTableBody');
        prepareProfitCharts(monthlyStats, 'monthlyProfitsChart', theme);
        prepareEarnedVsSpentCharts(monthlyStats, 'monthlySpentEarnedChart', theme);
        prepareBuysVsSalesCharts(monthlyStats, 'monthlyBuySalesChart', theme);
        lastMonthlyStats = monthlyStats;
    }

    if (saleDistributionUnion.length !== data.extra.saleDistribution.length) {
        prepareSaleDistributionChart(data.extra.saleDistribution, 'saleDistribution');
        prepareSalesDistributionTable(data.extra.saleDistribution, 'salesDistributionTableBody');
        lastSaleDistribution = data.extra.saleDistribution;
    }

    if (dailyTCVStatsUnion.length !== data.extra.dailyTCVStats.length) {
        prepareTCVChart(data.extra.dailyTCVStats, 'TCVChart', theme);
        prepareTCVTable(data.extra.dailyTCVStats, 'tcvTableBody');
        lastDailyTCVStats = data.extra.dailyTCVStats;
    }

    if(salesStatsUnion.length !== salesLog.data.length) {
        prepareProfitablePairsChart(salesLog.data, theme);
        prepareProfitablePairsTable(salesLog.data, 'mostProfitablePairsTableBody');
        prepareProfitablePairsTable(salesLog.data, 'leastProfitablePairsTableBody');
        prepareProfitablePairsTable(salesLog.data, 'mostSoldPairsTableBody');
        prepareDcaCounterTable(salesLog.data, 'dcaCountTableBody');
        prepareDcaCounterChart(salesLog.data, theme);
        tradeDuration(salesLog.data, theme);
        lastSellLogData = salesLog.data;
    }
}

function prepareTCVChart(data, element, theme) {
    let labels = [];
    let minTCV = [];

    let sortedData = _.orderBy(data, 'order', 'asc');

    for (let i = 0; i < sortedData.length; i++) {
        let date = changeDateFormat(sortedData[i].date);
        labels.push(date);

        minTCV.push(parseFloat((sortedData[i].minTCV).toFixed(8)));
    }

    new Chart(document.getElementById(element),
        {
            "type": "line",
            "data": {
                "labels": labels,
                "datasets": [
                    {
                        "label": i18next.t('statsPage.minTCV'),
                        "data": minTCV,
                        "fill": false,
                        "borderColor": "rgb(75, 192, 192)",
                        "lineTension": 0.4,
                        pointStyle: 'line'
                    }
                ],
            },
            "options": {
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": true,
                    labels: {
                        usePointStyle: true
                    }
                },
                "animation": false,
                "tooltips": {
                    "mode": "index",
                    "intersect": false,
                },
                "hover": {
                    "mode": "index",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor": theme === 'light' ? '#696969' : "rgba(152,166,173,0.5)",
                            "beginAtZero": false,
                            callback: function (value, index, values) {
                                return value.toFixed(8).toString()
                            },
                            suggestedMin: _.min(minTCV) - (0.01 / _.min(minTCV)),
                        },
                        "gridLines": {
                            "display": true,
                            "color": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                            "zeroLineColor": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor": theme === 'light' ? '#696969' : "rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,

                        },
                        "gridLines": {
                            "display": true,
                            "color": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                        }
                    }]
                }
            }
        });
}

function prepareSaleDistributionChart(data, element) {
    let labels = [], values = [];
    data.forEach(element => {
        labels.push(element.name);
        values.push(element.value);
    });

    new Chart(document.getElementById(element),
        {
            "type": "doughnut",
            "data": {
                "labels": labels,
                "datasets": [
                    {
                        "label": i18next.t('statsPage.salesDistribution'),
                        "data": values,
                        "backgroundColor": [
                            "rgb(5, 177, 111)",
                            "#96efbf",
                            "rgb(216, 83, 83)",
                            "#f66000",
                            "#6CB46C",
                            "#7bbbff",
                            "rgba(255, 215, 0, 0.75)"
                        ],
                        borderColor: [
                            "rgb(5, 177, 111)",
                            "#96efbf",
                            "rgb(216, 83, 83)",
                            "#f66000",
                            "#6CB46C",
                            "#7bbbff",
                            "rgba(255, 215, 0, 0.75)"
                        ]
                    }
                ]
            },
            "options": {
                "maintainAspectRatio": false,
                "plugins": {
                    "labels": [
                        {
                            "render": 'percentage',
                            "fontColor": "#000",
                            "precision": 0,
                            "showZero": false,
                            showActualPercentages: true,
                        }
                    ]
                },
                legend: {
                    display: true,
                    position: "right",
                    fullWidth: false,
                    labels: {
                        boxWidth: 1,
                        padding: 10,
                        fontSize: 13
                    }
                },
                "animation": false
            }
        });
}

function prepareBuysVsSalesCharts(sales, element, theme) {
    let labels = [];
    let sells = [];
    let buys = [];

    let sortedData = _.orderBy(sales, 'order', 'desc');

    for (let i = 0; i < sortedData.length; i++) {
        let date;
        if (element === 'monthlyBuySalesChart') {
            date = sortedData[i].month.split('-').join('/');
        } else {
            date = changeDateFormat(sortedData[i].date)
        }
        labels.push(date);

        sells.push((sortedData[i].totalSales));
        buys.push((sortedData[i].totalBuys));
    }

    new Chart(document.getElementById(element),
        {
            "type": "bar",
            "data": {
                "labels": labels,
                "datasets": [
                    {
                        "label": i18next.t('statsPage.buys'),
                        "data": buys,
                        "backgroundColor": "rgb(75, 192, 192)",
                        "borderColor": "rgb(75, 192, 192)",
                        pointStyle: 'line'
                    },
                    {
                        "label": i18next.t('statsPage.sales'),
                        "data": sells,
                        "backgroundColor": "#ef5350",
                        "borderColor": "#ef5350",
                        pointStyle: 'line'
                    }
                ]
            },
            "options": {
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": true,
                    labels: {
                        usePointStyle: true
                    }
                },
                "animation": false,
                "tooltips": {},
                "hover": {
                    "mode": "nearest",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor": theme === 'light' ? '#696969' : "rgba(152,166,173,0.5)",
                            beginAtZero: true
                        },
                        "gridLines": {
                            "display": true,
                            "color": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                            "zeroLineColor": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor": theme === 'light' ? '#696969' : "rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,
                        },
                        "gridLines": {
                            "display": true,
                            "color": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                        }
                    }]
                },
                plugins: {
                    labels: false
                }
            }
        });
}

function prepareEarnedVsSpentCharts(sales, element, theme) {
    let labels = [];
    let spent = [];
    let earned = [];

    let sortedData = _.orderBy(sales, 'order', 'desc');

    for (var i = 0; i < sortedData.length; i++) {
        let date;
        if (element === 'monthlySpentEarnedChart') {
            date = sortedData[i].month.split('-').join('/');
        } else {
            date = changeDateFormat(sortedData[i].date)
        }
        labels.push(date);

        spent.push(parseFloat((sortedData[i].totalBoughtCost).toFixed(8)));
        earned.push(parseFloat((sortedData[i].totalSoldValue).toFixed(8)));
    }

    new Chart(document.getElementById(element),
        {
            "type": "bar",
            "data": {
                "labels": labels,
                "datasets": [
                    {
                        "label": i18next.t('statsPage.spent'),
                        "data": spent,
                        "backgroundColor": "rgb(75, 192, 192)",
                        "borderColor": "rgb(75, 192, 192)",
                        pointStyle: 'line'
                    },
                    {
                        "label": i18next.t('statsPage.earned'),
                        "data": earned,
                        "backgroundColor": "#ef5350",
                        "borderColor": "#ef5350",
                        pointStyle: 'line'
                    }
                ]
            },
            "options": {
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": true,
                    labels: {
                        usePointStyle: true
                    }
                },
                "animation": false,
                "tooltips": {},
                "hover": {
                    "mode": "nearest",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor": theme === 'light' ? '#696969' : "rgba(152,166,173,0.5)",
                            beginAtZero: true,
                            callback: function (value, index, values) {
                                return value.toFixed(8).toString()
                            }
                        },
                        "gridLines": {
                            "display": true,
                            "color": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                            "zeroLineColor": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor": theme === 'light' ? '#696969' : "rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,
                        },
                        "gridLines": {
                            "display": true,
                            "color": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                        }
                    }]
                },
                plugins: {
                    labels: false
                }
            }
        });
}

function prepareProfitCharts(sales, element, theme) {
    let labels = [];
    let profits = [];

    let sortedData = _.orderBy(sales, 'order', 'desc');

    for (let i = 0; i < sortedData.length; i++) {
        let date;
        if (element === 'monthlyProfitsChart') {
            date = sortedData[i].month.replace('-', '/');
        } else {
            date = changeDateFormat(sortedData[i].date)
        }
        labels.push(date);

        profits.push(parseFloat((sortedData[i].totalProfitCurrency).toFixed(8)));
    }

    new Chart(document.getElementById(element),
        {
            "type": "line",
            "data": {
                "labels": labels,
                "datasets": [
                    {
                        "label": i18next.t('statsPage.profits'),
                        "data": profits,
                        "fill": false,
                        "borderColor": "rgb(75, 192, 192)",
                        "lineTension": 0.4,
                        pointStyle: 'line'
                    }
                ]
            },
            "options": {
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": true,
                    labels: {
                        usePointStyle: true
                    }
                },
                "animation": false,
                "tooltips": {
                    "mode": "nearest",
                    "intersect": false,
                },
                "hover": {
                    "mode": "nearest",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor": theme === 'light' ? '#696969' : "rgba(152,166,173,0.5)",
                            "beginAtZero": true,
                            callback: function (value, index, values) {
                                return value.toFixed(8).toString()
                            }
                        },
                        "gridLines": {
                            "display": true,
                            "color": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                            "zeroLineColor": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor": theme === 'light' ? '#696969' : "rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,

                        },
                        "gridLines": {
                            "display": true,
                            "color": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                        }
                    }]
                }
            }
        });

}


$("#swalSaleDistribution").on("click", swalSaleDistribution);

function swalSaleDistribution() {
    let content = document.createElement('div');
    content.innerHTML = 'TP = ' + i18next.t('statsPage.takeProfit') + '<br>' +
        'TPS = ' + i18next.t('statsPage.takeProfitSafety') + '<br>' +
        'TSL = ' + i18next.t('statsPage.trailingStopLoss') + '<br>' +
        'M = ' + i18next.t('statsPage.manual') + '<br>' +
        'MP = ' + i18next.t('statsPage.maxProfit') + '<br>' +
        'G = ' + i18next.t('statsPage.gain') + '<br>';

    Swal.fire({
        title: i18next.t('statsPage.salesDistributionAbbr'),
        html: content,
    })
}

function prepareDcaCounterChart(data, theme) {
    let grouped = _.groupBy(data, 'boughtTimes');
    let labels = [];
    let dcaCount = [];

    for(let count in grouped) {
        labels.push(grouped[count][0]['boughtTimes']);
        dcaCount.push(grouped[count].length);
    }
    labels = _.take(labels, 10);
    dcaCount = _.take(dcaCount, 10);

    new Chart(document.getElementById("dcaCounter"),
        {
            "type": "bar",
            "data": {
                "labels": labels,
                "datasets": [
                    {
                        "label": i18next.t('statsPage.sales'),
                        "data": dcaCount,
                        "backgroundColor": "rgb(75, 192, 192)",
                        "borderColor": "rgb(75, 192, 192)",
                        pointStyle: 'line'
                    }
                ]
            },
            "options": {
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": false,
                    labels: {
                        usePointStyle: true
                    }
                },
                "animation": false,
                "tooltips": {},
                "hover": {
                    "mode": "nearest",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor": theme === 'light' ? '#696969' : "rgba(152,166,173,0.5)",
                            beginAtZero: true,
                            callback: function (value, index, values) {
                                return value.toFixed(0).toString()
                            }
                        },
                        "gridLines": {
                            "display": true,
                            "color": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                            "zeroLineColor": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor": theme === 'light' ? '#696969' : "rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,
                        },
                        "gridLines": {
                            "display": true,
                            "color": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                        }
                    }]
                },
                plugins: {
                    labels: false
                }
            }
        });
}


function prepareProfitablePairsChart(sales, theme) {

    let groupSalesByMarket = _.groupBy(sales, 'market');
    let obj = {};

    for (let pair in groupSalesByMarket) {
        let profitCurrency = '0';
        let soldTimes = 0;


        for (let i = 0; i < groupSalesByMarket[pair].length; i++) {
            profitCurrency = parseFloat(profitCurrency) + parseFloat(groupSalesByMarket[pair][i].profitCurrency)
            soldTimes = soldTimes + 1
        }

        obj[groupSalesByMarket[pair][0]['market']] = {
            'profitCurrency': profitCurrency,
            'soldTimes': soldTimes,
            'coin': groupSalesByMarket[pair][0]['market']
        }
    }

    let sortedMostProfitable = sortDataProfitablePairs(obj, 'profitCurrency', 'desc', '10');
    let sortedLeastProfitable = sortDataProfitablePairs(obj, 'profitCurrency', 'asc', '10');
    let sortedMostSold = sortDataProfitablePairs(obj, 'soldTimes', 'desc', '10');

    ProfitablePairsChartRender(sortedMostSold[0], sortedMostSold[1], "mostSoldPairs", true, theme);
    ProfitablePairsChartRender(sortedMostProfitable[0], sortedMostProfitable[1], "mostProfitablePairs", theme);
    ProfitablePairsChartRender(sortedLeastProfitable[0], sortedLeastProfitable[1], "leastProfitablePairs", theme);
}

function sortDataProfitablePairs(data, field, direction, amount) {
    let sorted = _.take(_.orderBy(data, field, direction), amount);
    let l = []; // labels
    let d = []; // data

    for (j = 0; j < sorted.length; j++) {
        l.push(sorted[j]['coin']);
        d.push(sorted[j][field].toFixed(field === 'soldTimes' ? 0 : 8));
    }

    return [l, d]
}

function ProfitablePairsChartRender(labels, data, element, mostSoldChart = false, theme) {
    new Chart(document.getElementById(element),
        {
            "type": "bar",
            "data": {
                "labels": labels,
                "datasets": [
                    {
                        "label": mostSoldChart !== true ? i18next.t('statsPage.profit') : i18next.t('statsPage.timesSold'),
                        "data": data,
                        "backgroundColor": element === "leastProfitablePairs" ? "#ef5350" : "rgb(75, 192, 192)",
                        "borderColor": element === "leastProfitablePairs" ? "#ef5350" : "rgb(75, 192, 192)",
                        pointStyle: 'line'
                    }
                ]
            },
            "options": {
                "responsive": true,
                "maintainAspectRatio": false,
                "legend": {
                    "display": false,
                    labels: {
                        usePointStyle: true
                    }
                },
                "animation": false,
                "tooltips": {},
                "hover": {
                    "mode": "nearest",
                    "intersect": true,
                },
                "scales": {
                    "yAxes": [{
                        "ticks": {
                            "fontColor": theme === 'light' ? '#696969' : "rgba(152,166,173,0.5)",
                            beginAtZero: true,
                            callback: function (value, index, values) {
                                return value.toFixed(mostSoldChart === true ? 0 : 8).toString();
                            }
                        },
                        "gridLines": {
                            "display": true,
                            "color":  theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                            "zeroLineColor": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                        }
                    }],
                    "xAxes": [{
                        "ticks": {
                            "fontColor": theme === 'light' ? '#696969' : "rgba(152,166,173,0.5)",
                            "maxTicksLimit": 10,
                            autoSkip: false
                        },
                        "gridLines": {
                            "display": true,
                            "color": theme === 'light' ? '#D3D3D3' : "rgba(152,166,173,0.1)",
                        }
                    }]
                },
                plugins: {
                    labels: false
                }
            }
        });
}

function tradeDuration(sales) {
    let times = [];

    sales.forEach(function(pair) {
        let boughtDate = new Date(pair.firstBoughtDate * 1000);
        let soldDate = new Date(pair.soldDate * 1000);
        let difference = (soldDate - boughtDate) / 1000;
        times.push(difference)
    });

    let average = _.mean(times);
    document.getElementById("tradeDuration").innerHTML = (((average / 60) / 60)).toFixed(2);
}

/*
    Stats as Table Styles
 */
function prepareProfitTables(data, element) {

    let tableBody = $('#' + element);
    let tableProfitClass = '';
    tableBody.empty();

    let sortedData = _.orderBy(data, 'order', 'asc');

    for(let i = 0; i < sortedData.length; i++) {
        let date;
        if(element === 'monthlyProfitTableBody') {
            date = sortedData[i].month.replace('-', '/');
        } else {
            date = changeDateFormat(sortedData[i].date)
        }

        if(sortedData[i].totalProfitCurrency > 0) {
            tableProfitClass = 'profit-classtext'
        } else if (sortedData[i].totalProfitCurrency < 0) {
            tableProfitClass = 'loss-classtext'
        } else {
            tableProfitClass = ''
        }

        let tableRow = $("<tr></tr>")
            .append($("<td></td>").text(date))
            .append($("<td class='blue-color'></td>").text(sortedData[i].totalBuys))
            .append($("<td class='blue-color'></td>").text((sortedData[i].totalBoughtCost).toFixed(8)))
            .append($("<td class='blue-color'></td>").text(sortedData[i].totalSales))
            .append($("<td class='blue-color'></td>").text((sortedData[i].totalSoldValue).toFixed(8)));

        if(sortedData[i].totalProfitCurrency){
            tableRow.append($("<td class=" + tableProfitClass + "></td>").text((sortedData[i].totalProfitCurrency).toFixed(8)));
        }
        tableRow.appendTo(tableBody)
    }
}

function prepareTCVTable(data, element) {
    let tableBody = $('#' + element);
    let tableProfitClass = '';
    tableBody.empty();

    let sortedData = _.orderBy(data, 'order', 'desc');

    for(let i = 0; i < sortedData.length; i++) {
        let date = changeDateFormat(sortedData[i].date);

        if(sortedData[i].minTCV > 0) {
            tableProfitClass = 'profit-classtext'
        } else if (sortedData[i].minTCV < 0) {
            tableProfitClass = 'loss-classtext'
        } else {
            tableProfitClass = ''
        }

        let tableRow = $("<tr></tr>")
            .append($("<td></td>").text(date))
            .append($("<td class='"+ tableProfitClass +"'></td>").text((sortedData[i].minTCV).toFixed(8)));
        tableRow.appendTo(tableBody)
    }
}

function prepareSalesDistributionTable(data, element) {
    let tableBody = $('#' + element);
    tableBody.empty();

    for(let i = 0; i < data.length; i++) {
        let tableRow = $("<tr></tr>")
            .append($("<td></td>").text(data[i].name))
            .append($("<td class=''></td>").text((data[i].value)));
        tableRow.appendTo(tableBody);
    }
}

function prepareProfitablePairsTable(data, element) {
    let tableBody = $('#' + element);
    let sortedData, profitClass = '';
    tableBody.empty();

    let groupSalesByMarket = _.groupBy(data, 'market');
    let obj = {};

    for (let pair in groupSalesByMarket) {
        let profitCurrency = '0';
        let soldTimes = 0;


        for (let i = 0; i < groupSalesByMarket[pair].length; i++) {
            profitCurrency = parseFloat(profitCurrency) + parseFloat(groupSalesByMarket[pair][i].profitCurrency);
            soldTimes = soldTimes + 1
        }

        obj[groupSalesByMarket[pair][0]['market']] = {
            'profitCurrency': profitCurrency,
            'soldTimes': soldTimes,
            'coin': groupSalesByMarket[pair][0]['market']
        }
    }

    if(element === 'mostProfitablePairsTableBody') {
        sortedData = sortDataProfitablePairs(obj, 'profitCurrency', 'desc', '10');
    } else if(element === 'leastProfitablePairsTableBody') {
        sortedData = sortDataProfitablePairs(obj, 'profitCurrency', 'asc', '10');
    } else {
        sortedData = sortDataProfitablePairs(obj, 'soldTimes', 'desc', '10');
    }

    for(let i = 0; i < sortedData[0].length; i++) {
        if(element === 'mostProfitablePairsTableBody' || element === 'leastProfitablePairsTableBody') {
            profitClass = sortedData[1][i] > 0 ? 'profit-classtext' : 'loss-classtext';
        }
        let tableRow = $("<tr></tr>")
            .append($("<td class='yellow'></td>").text(sortedData[0][i]))
            .append($("<td class='"+profitClass+"'></td>").text((sortedData[1][i])));
        tableRow.appendTo(tableBody);
    }
}

function prepareDcaCounterTable(data, element) {
    let tableBody = $('#' + element);
    tableBody.empty();

    let grouped = _.groupBy(data, 'boughtTimes');

    for (let property in grouped) {
        let tableRow = $("<tr></tr>")
            .append($("<td></td>").text(property))
            .append($("<td class=''></td>").text((grouped[property].length)));
        tableRow.appendTo(tableBody);
    }
}

function changeDateFormat(date) {
    let formattedDate;
    formattedDate = date.split('-').join('/');
    formattedDate = formattedDate.split('/');
    formattedDate = formattedDate[1] + '/' + formattedDate[0] + '/' + formattedDate[2];
    formattedDate = DateTimeHelper.formatDate(new Date(formattedDate));
    formattedDate = formattedDate.split(' ')[0];
    return formattedDate;
}